from __future__ import unicode_literals
from collections import OrderedDict

from ..en import Provider as AddressProvider


class Provider(AddressProvider):
    city_prefixes = ('North', 'East', 'West', 'South', 'New', 'Lake', 'Port')

    city_suffixes = (
        'town',
        'ton',
        'land',
        'ville',
        'berg',
        'burgh',
        'borough',
        'bury',
        'view',
        'port',
        'mouth',
        'stad',
        'furt',
        'chester',
        'mouth',
        'fort',
        'haven',
        'side',
        'shire')

    building_number_formats = ('#####', '####', '###')

    street_suffixes = (
        'Alley',
        'Avenue',
        'Branch',
        'Bridge',
        'Brook',
        'Brooks',
        'Burg',
        'Burgs',
        'Bypass',
        'Camp',
        'Canyon',
        'Cape',
        'Causeway',
        'Center',
        'Centers',
        'Circle',
        'Circles',
        'Cliff',
        'Cliffs',
        'Club',
        'Common',
        'Corner',
        'Corners',
        'Course',
        'Court',
        'Courts',
        'Cove',
        'Coves',
        'Creek',
        'Crescent',
        'Crest',
        'Crossing',
        'Crossroad',
        'Curve',
        'Dale',
        'Dam',
        'Divide',
        'Drive',
        'Drive',
        'Drives',
        'Estate',
        'Estates',
        'Expressway',
        'Extension',
        'Extensions',
        'Fall',
        'Falls',
        'Ferry',
        'Field',
        'Fields',
        'Flat',
        'Flats',
        'Ford',
        'Fords',
        'Forest',
        'Forge',
        'Forges',
        'Fork',
        'Forks',
        'Fort',
        'Freeway',
        'Garden',
        'Gardens',
        'Gateway',
        'Glen',
        'Glens',
        'Green',
        'Greens',
        'Grove',
        'Groves',
        'Harbor',
        'Harbors',
        'Haven',
        'Heights',
        'Highway',
        'Hill',
        'Hills',
        'Hollow',
        'Inlet',
        'Inlet',
        'Island',
        'Island',
        'Islands',
        'Islands',
        'Isle',
        'Isle',
        'Junction',
        'Junctions',
        'Key',
        'Keys',
        'Knoll',
        'Knolls',
        'Lake',
        'Lakes',
        'Land',
        'Landing',
        'Lane',
        'Light',
        'Lights',
        'Loaf',
        'Lock',
        'Locks',
        'Locks',
        'Lodge',
        'Lodge',
        'Loop',
        'Mall',
        'Manor',
        'Manors',
        'Meadow',
        'Meadows',
        'Mews',
        'Mill',
        'Mills',
        'Mission',
        'Mission',
        'Motorway',
        'Mount',
        'Mountain',
        'Mountain',
        'Mountains',
        'Mountains',
        'Neck',
        'Orchard',
        'Oval',
        'Overpass',
        'Park',
        'Parks',
        'Parkway',
        'Parkways',
        'Pass',
        'Passage',
        'Path',
        'Pike',
        'Pine',
        'Pines',
        'Place',
        'Plain',
        'Plains',
        'Plains',
        'Plaza',
        'Plaza',
        'Point',
        'Points',
        'Port',
        'Port',
        'Ports',
        'Ports',
        'Prairie',
        'Prairie',
        'Radial',
        'Ramp',
        'Ranch',
        'Rapid',
        'Rapids',
        'Rest',
        'Ridge',
        'Ridges',
        'River',
        'Road',
        'Road',
        'Roads',
        'Roads',
        'Route',
        'Row',
        'Rue',
        'Run',
        'Shoal',
        'Shoals',
        'Shore',
        'Shores',
        'Skyway',
        'Spring',
        'Springs',
        'Springs',
        'Spur',
        'Spurs',
        'Square',
        'Square',
        'Squares',
        'Squares',
        'Station',
        'Station',
        'Stravenue',
        'Stravenue',
        'Stream',
        'Stream',
        'Street',
        'Street',
        'Streets',
        'Summit',
        'Summit',
        'Terrace',
        'Throughway',
        'Trace',
        'Track',
        'Trafficway',
        'Trail',
        'Trail',
        'Tunnel',
        'Tunnel',
        'Turnpike',
        'Turnpike',
        'Underpass',
        'Union',
        'Unions',
        'Valley',
        'Valleys',
        'Via',
        'Viaduct',
        'View',
        'Views',
        'Village',
        'Village',
        'Villages',
        'Ville',
        'Vista',
        'Vista',
        'Walk',
        'Walks',
        'Wall',
        'Way',
        'Ways',
        'Well',
        'Wells')

    postcode_formats = ('#####', '#####-####')

    states = (
        'Alabama', 'Alaska', 'Arizona', 'Arkansas', 'California', 'Colorado',
        'Connecticut', 'Delaware', 'Florida', 'Georgia', 'Hawaii', 'Idaho',
        'Illinois', 'Indiana', 'Iowa', 'Kansas', 'Kentucky', 'Louisiana',
        'Maine', 'Maryland', 'Massachusetts', 'Michigan', 'Minnesota',
        'Mississippi', 'Missouri', 'Montana', 'Nebraska', 'Nevada',
        'New Hampshire', 'New Jersey', 'New Mexico', 'New York',
        'North Carolina', 'North Dakota', 'Ohio', 'Oklahoma', 'Oregon',
        'Pennsylvania', 'Rhode Island', 'South Carolina', 'South Dakota',
        'Tennessee', 'Texas', 'Utah', 'Vermont', 'Virginia', 'Washington',
        'West Virginia', 'Wisconsin', 'Wyoming',
    )
    states_abbr = (
        'AL', 'AK', 'AZ', 'AR', 'CA', 'CO', 'CT', 'DE', 'DC', 'FL', 'GA', 'HI',
        'ID', 'IL', 'IN', 'IA', 'KS', 'KY', 'LA', 'ME', 'MD', 'MA', 'MI', 'MN',
        'MS', 'MO', 'MT', 'NE', 'NV', 'NH', 'NJ', 'NM', 'NY', 'NC', 'ND', 'OH',
        'OK', 'OR', 'PA', 'RI', 'SC', 'SD', 'TN', 'TX', 'UT', 'VT', 'VA', 'WA',
        'WV', 'WI', 'WY',
    )

    states_postcode = {
        'AL': (35004, 36925), 'AK': (99501, 99950), 'AZ': (85001, 86556),
        'AR': (71601, 72959), 'CA': (90001, 96162), 'CO': (80001, 81658),
        'CT': (6001, 6389), 'DE': (19701, 19980), 'DC': (20001, 20039),
        'FL': (32004, 34997), 'GA': (30001, 31999), 'HI': (96701, 96898),
        'ID': (83201, 83876), 'IL': (60001, 62999), 'IN': (46001, 47997),
        'IA': (50001, 52809), 'KS': (66002, 67954), 'KY': (40003, 42788),
        'LA': (70001, 71232), 'ME': (3901, 4992), 'MD': (20331, 20331),
        'MA': (1001, 2791), 'MI': (48001, 49971), 'MN': (55001, 56763),
        'MS': (38601, 39776), 'MO': (63001, 65899), 'MT': (59001, 59937),
        'NE': (68001, 68118), 'NV': (88901, 89883), 'NH': (3031, 3897),
        'NJ': (7001, 8989), 'NM': (87001, 88441), 'NY': (10001, 14905),
        'NC': (27006, 28909), 'ND': (58001, 58856), 'OH': (43001, 45999),
        'OK': (73001, 73199), 'OR': (97001, 97920), 'PA': (15001, 19640),
        'RI': (2801, 2940), 'SC': (29001, 29948), 'SD': (57001, 57799),
        'TN': (37010, 38589), 'TX': (73301, 73301), 'UT': (84001, 84784),
        'VT': (5001, 5495), 'VA': (20040, 20041), 'WA': (98001, 99403),
        'WV': (24701, 26886), 'WI': (53001, 54990), 'WY': (82001, 83128),
    }

    territories_abbr = (
        'AS', 'FM', 'GU', 'MH', 'MP', 'PW', 'PR', 'VI',
    )

    states_and_territories_abbr = states_abbr + territories_abbr

    military_state_abbr = ('AE', 'AA', 'AP')

    military_ship_prefix = ('USS', 'USNS', 'USNV', 'USCGC')

    military_apo_format = ("PSC ####, Box ####")

    military_dpo_format = ("Unit #### Box ####")

    city_formats = (
        '{{city_prefix}} {{first_name}}{{city_suffix}}',
        '{{city_prefix}} {{first_name}}',
        '{{first_name}}{{city_suffix}}',
        '{{last_name}}{{city_suffix}}',
    )

    street_name_formats = (
        '{{first_name}} {{street_suffix}}',
        '{{last_name}} {{street_suffix}}',
    )

    street_address_formats = (
        '{{building_number}} {{street_name}}',
        '{{building_number}} {{street_name}} {{secondary_address}}',
    )

    address_formats = (
        "{{street_address}}\n{{city}}, {{state_abbr}} {{postcode}}",
    )

    address_formats = OrderedDict((
        ("{{street_address}}\n{{city}}, {{state_abbr}} {{postcode}}", 25),
        #  military address formatting.
        ("{{military_apo}}\nAPO {{military_state}} {{postcode}}", 1),
        ("{{military_ship}} {{last_name}}\nFPO {{military_state}} {{postcode}}", 1),
        ("{{military_dpo}}\nDPO {{military_state}} {{postcode}}", 1),
    ))

    secondary_address_formats = ('Apt. ###', 'Suite ###')

    def city_prefix(self):
        return self.random_element(self.city_prefixes)

    def secondary_address(self):
        return self.numerify(
            self.random_element(
                self.secondary_address_formats))

    def state(self):
        return self.random_element(self.states)

    def state_abbr(self, include_territories=True):
        """
        :returns: A random state or territory abbreviation.

        :param include_territories: If True, territories will be included.
            If False, only states will be returned.
        """
        if include_territories:
            self.random_element(self.states_and_territories_abbr)
        return self.random_element(self.states_abbr)

    def postcode(self):
        return "%05d" % self.generator.random.randint(501, 99950)

    def zipcode_plus4(self):
        return "%s-%04d" % (self.zipcode(),
                            self.generator.random.randint(1, 9999))

    def postcode_in_state(self, state_abbr=None):
        """
        :returns: A random postcode within the provided state abbreviation

        :param state_abbr: A state abbreviation
        """
        if state_abbr is None:
            state_abbr = self.random_element(self.states_abbr)

        if state_abbr in self.states_abbr:
            postcode = "%d" % (self.generator.random.randint(
                            self.states_postcode[state_abbr][0],
                            self.states_postcode[state_abbr][1]))

            if len(postcode) == 4:
                postcode = "0%s" % postcode

            return postcode

        else:
            raise Exception('State Abbreviation not found in list')

    def military_ship(self):
        """
        :example 'USS'
        """
        return self.random_element(self.military_ship_prefix)

    def military_state(self):
        """
        :example 'APO'
        """
        return self.random_element(self.military_state_abbr)

    def military_apo(self):
        """
        :example 'PSC 5394 Box 3492
        """
        return self.numerify(self.military_apo_format)

    def military_dpo(self):
        """
        :example 'Unit 3333 Box 9342'
        """
        return self.numerify(self.military_dpo_format)

    # Aliases
    def zipcode(self):
        return self.postcode()

    def zipcode_in_state(self, state_abbr=None):
        return self.postcode_in_state(state_abbr)

    def postalcode(self):
        return self.postcode()

    def postalcode_in_state(self, state_abbr=None):
        return self.postcode_in_state(state_abbr)

    def postalcode_plus4(self):
        return self.zipcode_plus4()



# coding=utf-8
from __future__ import unicode_literals
from collections import OrderedDict

from .. import Provider as ColorProvider


class Provider(ColorProvider):
    all_colors = OrderedDict((
        ("أزرق أليس", "#F0F8FF"),
        ("أبيض عتيق", "#FAEBD7"),
        ("مائي", "#00FFFF"),
        ("زبرجدي", "#7FFFD4"),
        ("لازوردي", "#F0FFFF"),
        ("صوفي", "#F5F5DC"),
        ("حسائي", "#FFE4C4"),
        ("أسود", "#000000"),
        ("لوزي", "#FFEBCD"),
        ("أزرق", "#0000FF"),
        ("بنفسجي مزرق", "#8A2BE2"),
        ("بني", "#A52A2A"),
        ("خشبية", "#DEB887"),
        ("أزرق كاديتي", "#5F9EA0"),
        ("كرتوزي", "#7FFF00"),
        ("شوكولاتيّ", "#D2691E"),
        ("مرجاني", "#FF7F50"),
        ("قنطريوني", "#6495ED"),
        ("حرير الذرة", "#FFF8DC"),
        ("قرمزي", "#DC143C"),
        ("سيان", "#00FFFF"),
        ("أزرق داكن", "#00008B"),
        ("سيان داكن", "#008B8B"),
        ("عصا الدهب الغامق", "#B8860B"),
        ("رمادي داكن", "#A9A9A9"),
        ("أخضر داكن", "#006400"),
        ("خاكي داكن", "#BDB76B"),
        ("ماجنتا داكن", "#8B008B"),
        ("أخضر زيتوني داكن", "#556B2F"),
        ("برتقالي داكن", "#FF8C00"),
        ("أوركيدي داكن", "#9932CC"),
        ("أحمر داكن", "#8B0000"),
        ("سلموني داكن", "#E9967A"),
        ("أخضر بحري داكن", "#8FBC8F"),
        ("أزرق أردوازي داكن", "#483D8B"),
        ("رمادي لازوردي داكن", "#2F4F4F"),
        ("تركوازي داكن", "#00CED1"),
        ("بنفسج داكن", "#9400D3"),
        ("زهري غامق", "#FF1493"),
        ("أزرق سماوي غامق", "#00BFFF"),
        ("رمادي خافت", "#696969"),
        ("أزرق فريق دودجر", "#1E90FF"),
        ("الطوب شمت", "#B22222"),
        ("أبيض وردي", "#FFFAF0"),
        ("أخضر الغابت", "#228B22"),
        ("فوشي", "#FF00FF"),
        ("رمادي باهت", "#DCDCDC"),
        ("أبيض شبحي", "#F8F8FF"),
        ("ذهبي", "#FFD700"),
        ("ذهبي", "#DAA520"),
        ("رمادي", "#808080"),
        ("أخضر", "#008000"),
        ("أصفر مخضر", "#ADFF2F"),
        ("عسلي", "#F0FFF0"),
        ("وردي فاقع", "#FF69B4"),
        ("قسطلي", "#CD5C5C"),
        ("نيلي", "#4B0082"),
        ("سكري", "#FFFFF0"),
        ("خاكي", "#F0E68C"),
        ("لاڤندر", "#E6E6FA"),
        ("أحمر اللافندر", "#FFF0F5"),
        ("أخضر عشبي", "#7CFC00"),
        ("ليمون شيفوني", "#FFFACD"),
        ("أزرق فاتح", "#ADD8E6"),
        ("مرجاني فاتح", "#F08080"),
        ("أزرق طفولي", "#E0FFFF"),
        ("أصفر ذهبي فاتح ", "#FAFAD2"),
        ("رمادي فاتح", "#D3D3D3"),
        ("أخضر فاتح", "#90EE90"),
        ("وردي فاتح", "#FFB6C1"),
        ("سلموني فاتح", "#FFA07A"),
        ("أخضر بحري فاتح", "#20B2AA"),
        ("سماوي فاتح", "#87CEFA"),
        ("أزرق أردوازي فاتح", "#778899"),
        ("أزرق معدني فاتح", "#B0C4DE"),
        ("أصفر فاتح", "#FFFFE0"),
        ("ليمي", "#00FF00"),
        ("أخضر ليموني", "#32CD32"),
        ("كتاني", "#FAF0E6"),
        ("فوشيا", "#FF00FF"),
        ("كستنائي", "#800000"),
        ("زبرجدي متوسط", "#66CDAA"),
        ("أزرق متوسط", "#0000CD"),
        ("أوركيدي متوسط", "#BA55D3"),
        ("فوشي متوسط", "#9370DB"),
        ("أخضر بحري متوسط", "#3CB371"),
        ("أزرق أردوازي متوسط", "#7B68EE"),
        ("أخضر ربيعي متوسط", "#00FA9A"),
        ("ترموازي متوسط", "#48D1CC"),
        ("أحمر بنفسجي", "#C71585"),
        ("الأزرق متوسط", "#191970"),
        ("نعناعي كريمي", "#F5FFFA"),
        ("الوردي الضبابي", "#FFE4E1"),
        ("موكاسيني", "#FFE4B5"),
        ("أبيض نافاجو", "#FFDEAD"),
        ("كحلي", "#000080"),
        ("رباطي قديم", "#FDF5E6"),
        ("زيتوني", "#808000"),
        ("زيتوني رمادي", "#6B8E23"),
        ("برتقالي", "#FFA500"),
        ("أحمر برتقالي", "#FF4500"),
        ("أوركيدي", "#DA70D6"),
        ("ذهبي باهت", "#EEE8AA"),
        ("أخضر باهت", "#98FB98"),
        ("تركوازي باهت", "#AFEEEE"),
        ("أحمر بنفسجي باهت", "#DB7093"),
        ("بابايا", "#FFEFD5"),
        ("حنطي", "#FFDAB9"),
        ("بيرو", "#CD853F"),
        ("زهري", "#FFC0CB"),
        ("برقوقي", "#DDA0DD"),
        ("أزرق مسحوقي", "#B0E0E6"),
        ("أرجواني", "#800080"),
        ("أحمر", "#FF0000"),
        ("بني وردي", "#BC8F8F"),
        ("أزرق ملكي", "#4169E1"),
        ("بني السرج", "#8B4513"),
        ("سالموني", "#FA8072"),
        ("بني رملي", "#F4A460"),
        ("أخضر بحري", "#2E8B57"),
        ("صدفي", "#FFF5EE"),
        ("سيينا", "#A0522D"),
        ("فضي", "#C0C0C0"),
        ("أزرق سماي", "#87CEEB"),
        ("أزرق أردوازي", "#6A5ACD"),
        ("رمادي معدني", "#708090"),
        ("ثلجي", "#FFFAFA"),
        ("أخضر ربيعي", "#00FF7F"),
        ("أزرق معدني", "#4682B4"),
        ("نطي", "#D2B48C"),
        ("حذفي", "#008080"),
        ("أرجواني", "#D8BFD8"),
        ("طماطمي", "#FF6347"),
        ("تركواز", "#40E0D0"),
        ("بنفسجي", "#EE82EE"),
        ("قمحي", "#F5DEB3"),
        ("أبيض", "#FFFFFF"),
        ("دخاني قمحي", "#F5F5F5"),
        ("أصفر", "#FFFF00"),
        ("أصفر مخضر", "#9ACD32"),
    ))

    safe_colors = (
        'أسود', 'كستنائي', 'أخضر', 'كحلي', 'زيتوني',
        'أرجواني', 'حذفي', 'ليمي', 'أزرق', 'فضي',
        'رمادي', 'أصفر', 'فوشي', 'مائي', 'أبيض',
    )



# coding=utf-8

from __future__ import unicode_literals
import hashlib
import string
import uuid
import sys

from .. import BaseProvider


localized = True


class Provider(BaseProvider):
    def boolean(self, chance_of_getting_true=50):
        return self.generator.random.randint(1, 100) <= chance_of_getting_true

    def null_boolean(self):
        return {
            0: None,
            1: True,
            -1: False,
        }[self.generator.random.randint(-1, 1)]

    def binary(self, length=(1 * 1024 * 1024)):
        """ Returns random binary blob.

        Default blob size is 1 Mb.
        """
        blob = [self.generator.random.randrange(256) for _ in range(length)]
        return bytes(blob) if sys.version_info[0] >= 3 else bytearray(blob)

    def md5(self, raw_output=False):
        """
        Calculates the md5 hash of a given string
        :example 'cfcd208495d565ef66e7dff9f98764da'
        """
        res = hashlib.md5(str(self.generator.random.random()).encode('utf-8'))
        if raw_output:
            return res.digest()
        return res.hexdigest()

    def sha1(self, raw_output=False):
        """
        Calculates the sha1 hash of a given string
        :example 'b5d86317c2a144cd04d0d7c03b2b02666fafadf2'
        """
        res = hashlib.sha1(str(self.generator.random.random()).encode('utf-8'))
        if raw_output:
            return res.digest()
        return res.hexdigest()

    def sha256(self, raw_output=False):
        """
        Calculates the sha256 hash of a given string
        :example '85086017559ccc40638fcde2fecaf295e0de7ca51b7517b6aebeaaf75b4d4654'
        """
        res = hashlib.sha256(
            str(self.generator.random.random()).encode('utf-8'))
        if raw_output:
            return res.digest()
        return res.hexdigest()

    def uuid4(self, cast_to=str):
        """
        Generates a random UUID4 string.
        :param cast_to: Specify what type the UUID should be cast to. Default is `str`
        :type cast_to: callable
        """
        # Based on http://stackoverflow.com/q/41186818
        return cast_to(uuid.UUID(int=self.generator.random.getrandbits(128), version=4))

    def password(
            self,
            length=10,
            special_chars=True,
            digits=True,
            upper_case=True,
            lower_case=True):
        """
        Generates a random password.
        @param length: Integer. Length of a password
        @param special_chars: Boolean. Whether to use special characters !@#$%^&*()_+
        @param digits: Boolean. Whether to use digits
        @param upper_case: Boolean. Whether to use upper letters
        @param lower_case: Boolean. Whether to use lower letters
        @return: String. Random password
        """
        choices = ""
        required_tokens = []
        if special_chars:
            required_tokens.append(
                self.generator.random.choice("!@#$%^&*()_+"))
            choices += "!@#$%^&*()_+"
        if digits:
            required_tokens.append(self.generator.random.choice(string.digits))
            choices += string.digits
        if upper_case:
            required_tokens.append(
                self.generator.random.choice(string.ascii_uppercase))
            choices += string.ascii_uppercase
        if lower_case:
            required_tokens.append(
                self.generator.random.choice(string.ascii_lowercase))
            choices += string.ascii_lowercase

        assert len(
            required_tokens) <= length, "Required length is shorter than required characters"

        # Generate a first version of the password
        chars = self.random_choices(choices, length=length)

        # Pick some unique locations
        random_indexes = set()
        while len(random_indexes) < len(required_tokens):
            random_indexes.add(
                self.generator.random.randint(0, len(chars) - 1))

        # Replace them with the required characters
        for i, index in enumerate(random_indexes):
            chars[index] = required_tokens[i]

        return ''.join(chars)



# coding=utf-8
from __future__ import unicode_literals
from ..la import Provider as LoremProvider
from ..en_US import Provider as EnUsProvider


class Provider(LoremProvider):
    """Provider for generating English or Latin lorem content"""

    english_word_list = EnUsProvider.word_list

    def english_word(self):
        return self.word(self.english_word_list)

    def english_words(self, nb=3, unique=False):
        return self.words(nb, self.english_word_list, unique)

    def english_sentence(self, nb_words=6, variable_nb_words=True):
        return self.sentence(nb_words, variable_nb_words, self.english_word_list)

    def english_sentences(self, nb=3):
        return self.sentences(nb, self.english_word_list)

    def english_paragraph(self, nb_sentences=3, variable_nb_sentences=True):
        return self.paragraph(nb_sentences, variable_nb_sentences, self.english_word_list)

    def english_paragraphs(self, nb=3):
        return self.paragraphs(nb, self.english_word_list)

    def english_text(self, max_nb_chars=200):
        return self.text(max_nb_chars, self.english_word_list)

    def english_texts(self, nb_texts=3, max_nb_chars=200):
        return self.texts(nb_texts, max_nb_chars, self.english_word_list)



# coding=utf-8
from __future__ import unicode_literals

from collections import OrderedDict

from .. import Provider as AddressProvider


class Provider(AddressProvider):
    city_suffixes = ['berg', 'borg', 'by', 'bø', 'dal', 'eid', 'fjell',
                     'fjord', 'foss', 'grunn', 'hamn', 'havn', 'helle', 'mark',
                     'nes', 'odden', 'sand', 'sjøen', 'stad', 'strand',
                     'strøm', 'sund', 'vik', 'vær', 'våg', 'ø', 'øy', 'ås']
    street_suffixes = ['alléen', 'bakken', 'berget', 'bråten', 'eggen',
                       'engen', 'ekra', 'faret', 'flata', 'gata', 'gjerdet',
                       'grenda', 'gropa', 'hagen', 'haugen', 'havna', 'holtet',
                       'høgda', 'jordet', 'kollen', 'kroken', 'lia', 'lunden',
                       'lyngen', 'løkka', 'marka', 'moen', 'myra', 'plassen',
                       'ringen', 'roa', 'røa', 'skogen', 'skrenten',
                       'spranget', 'stien', 'stranda', 'stubben', 'stykket',
                       'svingen', 'tjernet', 'toppen', 'tunet', 'vollen',
                       'vika', 'åsen']
    city_formats = [
        '{{first_name}}{{city_suffix}}', '{{last_name}}']
    street_name_formats = [
        '{{last_name}}{{street_suffix}}',
    ]
    street_address_formats = ('{{street_name}} {{building_number}}',)
    address_formats = ('{{street_address}}, {{postcode}} {{city}}',)
    building_number_formats = ('%', '%', '%', '%?', '##', '##', '##?', '###')
    building_number_suffixes = OrderedDict([
        ('A', 0.2),
        ('B', 0.2),
        ('C', 0.2),
        ('D', 0.1),
        ('E', 0.1),
        ('F', 0.1),
        ('G', 0.05),
        ('H', 0.05),
    ])
    postcode_formats = ('####',)

    def building_number(self):
        suffix = self.random_element(self.building_number_suffixes)
        return self.numerify(
            self.random_element(
                self.building_number_formats)).replace(
            '?', suffix)

    def city_suffix(self):
        return self.random_element(self.city_suffixes)

    def street_suffix(self):
        return self.random_element(self.street_suffixes)



# coding=utf-8

from __future__ import unicode_literals

from decimal import Decimal

from .. import Provider as GeoProvider


class Provider(GeoProvider):

    poly = (
        (40.34026, 19.15120),
        (42.21670, 26.13934),
        (35.55680, 29.38280),
        (34.15370, 22.58810),
    )

    def local_latlng(self):
        return float(self.local_latitude()), float(self.local_longitude())

    def local_latitude(self):
        latitudes = list(map(lambda t: int(t[0] * 10000000), self.poly))
        return Decimal(str(self.generator.random.randint(
            min(latitudes), max(latitudes)) / 10000000.0)).quantize(Decimal('.000001'))

    def local_longitude(self):
        longitudes = list(map(lambda t: int(t[1] * 10000000), self.poly))
        return Decimal(str(self.generator.random.randint(
            min(longitudes), max(longitudes)) / 10000000.0)).quantize(Decimal('.000001'))



# coding=utf-8
from __future__ import unicode_literals
from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats_female = (
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{prefix}} {{last_name}}',
        '{{first_name_female}} {{last_name}}-{{last_name}}',
        '{{first_name_female}}-{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}} {{prefix}} {{last_name}}',
    )

    formats_male = (
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{prefix}} {{last_name}}',
        '{{first_name_male}} {{last_name}}-{{last_name}}',
        '{{first_name_male}}-{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}} {{prefix}} {{last_name}}',
    )

    formats = formats_male + formats_female

    first_names_male = (
        'Adrien',
        'Aimé',
        'Alain',
        'Alexandre',
        'Alfred',
        'Alphonse',
        'André',
        'Antoine',
        'Arthur',
        'Auguste',
        'Augustin',
        'Benjamin',
        'Benoît',
        'Bernard',
        'Bertrand',
        'Charles',
        'Christophe',
        'Daniel',
        'David',
        'Denis',
        'Édouard',
        'Émile',
        'Emmanuel',
        'Éric',
        'Étienne',
        'Eugène',
        'François',
        'Franck',
        'Frédéric',
        'Gabriel',
        'Georges',
        'Gérard',
        'Gilbert',
        'Gilles',
        'Grégoire',
        'Guillaume',
        'Guy',
        'William',
        'Henri',
        'Honoré',
        'Hugues',
        'Isaac',
        'Jacques',
        'Jean',
        'Jérôme',
        'Joseph',
        'Jules',
        'Julien',
        'Laurent',
        'Léon',
        'Louis',
        'Luc',
        'Lucas',
        'Marc',
        'Marcel',
        'Martin',
        'Matthieu',
        'Maurice',
        'Michel',
        'Nicolas',
        'Noël',
        'Olivier',
        'Patrick',
        'Paul',
        'Philippe',
        'Pierre',
        'Raymond',
        'Rémy',
        'René',
        'Richard',
        'Robert',
        'Roger',
        'Roland',
        'Sébastien',
        'Stéphane',
        'Théodore',
        'Théophile',
        'Thibaut',
        'Thibault',
        'Thierry',
        'Thomas',
        'Timothée',
        'Tristan',
        'Victor',
        'Vincent',
        'Xavier',
        'Yves',
        'Zacharie')

    first_names_female = (
        'Adélaïde',
        'Adèle',
        'Adrienne',
        'Agathe',
        'Agnès',
        'Aimée',
        'Alexandrie',
        'Alix',
        'Alexandria',
        'Alex',
        'Alice',
        'Amélie',
        'Anaïs',
        'Anastasie',
        'Andrée',
        'Anne',
        'Anouk',
        'Antoinette',
        'Arnaude',
        'Astrid',
        'Audrey',
        'Aurélie',
        'Aurore',
        'Bernadette',
        'Brigitte',
        'Capucine',
        'Caroline',
        'Catherine',
        'Cécile',
        'Céline',
        'Célina',
        'Chantal',
        'Charlotte',
        'Christelle',
        'Christiane',
        'Christine',
        'Claire',
        'Claudine',
        'Clémence',
        'Colette',
        'Constance',
        'Corinne',
        'Danielle',
        'Denise',
        'Diane',
        'Dorothée',
        'Édith',
        'Éléonore',
        'Élisabeth',
        'Élise',
        'Élodie',
        'Émilie',
        'Emmanuelle',
        'Françoise',
        'Frédérique',
        'Gabrielle',
        'Geneviève',
        'Hélène',
        'Henriette',
        'Hortense',
        'Inès',
        'Isabelle',
        'Jacqueline',
        'Jeanne',
        'Jeannine',
        'Joséphine',
        'Josette',
        'Julie',
        'Juliette',
        'Laetitia',
        'Laure',
        'Laurence',
        'Lorraine',
        'Louise',
        'Luce',
        'Lucie',
        'Lucy',
        'Madeleine',
        'Manon',
        'Marcelle',
        'Margaux',
        'Margaud',
        'Margot',
        'Marguerite',
        'Margot',
        'Margaret',
        'Maggie',
        'daisy',
        'Marianne',
        'Marie',
        'Marine',
        'Marthe',
        'Martine',
        'Maryse',
        'Mathilde',
        'Michèle',
        'Michelle',
        'Michelle',
        'Monique',
        'Nathalie',
        'Nath',
        'Nathalie',
        'Nicole',
        'Noémi',
        'Océane',
        'Odette',
        'Olivie',
        'Patricia',
        'Paulette',
        'Pauline',
        'Pénélope',
        'Philippine',
        'Renée',
        'Sabine',
        'Simone',
        'Sophie',
        'Stéphanie',
        'Susanne',
        'Suzanne',
        'Susan',
        'Suzanne',
        'Sylvie',
        'Thérèse',
        'Valentine',
        'Valérie',
        'Véronique',
        'Victoire',
        'Virginie',
        'Zoé',
        'Camille',
        'Claude',
        'Dominique')

    first_names = first_names_male + first_names_female

    last_names = (
        'Martin', 'Bernard', 'Thomas', 'Robert', 'Petit', 'Dubois', 'Richard', 'Garcia', 'Durand', 'Moreau', 'Lefebvre',
        'Simon', 'Laurent', 'Michel', 'Leroy', 'Martinez', 'David', 'Fontaine', 'Da Silva', 'Morel', 'Fournier',
        'Dupont', 'Bertrand', 'Lambert', 'Rousseau', 'Girard', 'Roux', 'Vincent', 'Lefevre', 'Boyer', 'Lopez', 'Bonnet',
        'Andre', 'Francois', 'Mercier', 'Muller', 'Guerin', 'Legrand', 'Sanchez', 'Garnier', 'Chevalier', 'Faure',
        'Perez', 'Clement', 'Fernandez', 'Blanc', 'Robin', 'Morin', 'Gauthier', 'Pereira', 'Perrin', 'Roussel', 'Henry',
        'Duval', 'Gautier', 'Nicolas', 'Masson', 'Marie', 'Noel', 'Ferreira', 'Lemaire', 'Mathieu', 'Riviere', 'Denis',
        'Marchand', 'Rodriguez', 'Dumont', 'Payet', 'Lucas', 'Dufour', 'Dos Santos', 'Joly', 'Blanchard', 'Meunier',
        'Rodrigues', 'Caron', 'Gerard', 'Fernandes', 'Brunet', 'Meyer', 'Barbier', 'Leroux', 'Renard', 'Goncalves',
        'Gaillard', 'Brun', 'Roy', 'Picard', 'Giraud', 'Roger', 'Schmitt', 'Colin', 'Arnaud', 'Vidal', 'Gonzalez',
        'Lemoine', 'Roche', 'Aubert', 'Olivier', 'Leclercq', 'Pierre', 'Philippe', 'Bourgeois', 'Renaud', 'Martins',
        'Leclerc', 'Guillaume', 'Lacroix', 'Lecomte', 'Benoit', 'Fabre', 'Carpentier', 'Vasseur', 'Louis', 'Hubert',
        'Jean', 'Dumas', 'Rolland', 'Grondin', 'Rey', 'Huet', 'Gomez', 'Dupuis', 'Guillot', 'Berger', 'Moulin',
        'Hoarau', 'Menard', 'Deschamps', 'Fleury', 'Adam', 'Boucher', 'Poirier', 'Bertin', 'Charles', 'Aubry',
        'Da Costa', 'Royer', 'Dupuy', 'Maillard', 'Paris', 'Baron', 'Lopes', 'Guyot', 'Carre', 'Jacquet', 'Renault',
        'Herve', 'Charpentier', 'Klein', 'Cousin', 'Collet', 'Leger', 'Ribeiro', 'Hernandez', 'Bailly', 'Schneider',
        'Le Gall', 'Ruiz', 'Langlois', 'Bouvier', 'Gomes', 'Prevost', 'Julien', 'Lebrun', 'Breton', 'Germain', 'Millet',
        'Boulanger', 'Remy', 'Le Roux', 'Daniel', 'Marques', 'Maillot', 'Leblanc', 'Le Goff', 'Barre', 'Perrot',
        'Leveque', 'Marty', 'Benard', 'Monnier', 'Hamon', 'Pelletier', 'Alves', 'Etienne', 'Marchal', 'Poulain',
        'Tessier', 'Lemaitre', 'Guichard', 'Besson', 'Mallet', 'Hoareau', 'Gillet', 'Weber', 'Jacob', 'Collin',
        'Chevallier', 'Perrier', 'Michaud', 'Carlier', 'Delaunay', 'Chauvin', 'Alexandre', 'Marechal', 'Antoine',
        'Lebon', 'Cordier', 'Lejeune', 'Bouchet', 'Pasquier', 'Legros', 'Delattre', 'Humbert', 'De Oliveira', 'Briand',
        'Lamy', 'Launay', 'Gilbert', 'Perret', 'Lesage', 'Gay', 'Nguyen', 'Navarro', 'Besnard', 'Pichon', 'Hebert',
        'Cohen', 'Pons', 'Lebreton', 'Sauvage', 'De Sousa', 'Pineau', 'Albert', 'Jacques', 'Pinto', 'Barthelemy',
        'Turpin', 'Bigot', 'Lelievre', 'Georges', 'Reynaud', 'Ollivier', 'Martel', 'Voisin', 'Leduc', 'Guillet',
        'Vallee', 'Coulon', 'Camus', 'Marin', 'Teixeira', 'Costa', 'Mahe', 'Didier', 'Charrier', 'Gaudin', 'Bodin',
        'Guillou', 'Gregoire', 'Gros', 'Blanchet', 'Buisson', 'Blondel', 'Paul', 'Dijoux', 'Barbe', 'Hardy', 'Laine',
        'Evrard', 'Laporte', 'Rossi', 'Joubert', 'Regnier', 'Tanguy', 'Gimenez', 'Allard', 'Devaux', 'Morvan', 'Levy',
        'Dias', 'Courtois', 'Lenoir', 'Berthelot', 'Pascal', 'Vaillant', 'Guilbert', 'Thibault', 'Moreno', 'Duhamel',
        'Colas', 'Masse', 'Baudry', 'Bruneau', 'Verdier', 'Delorme', 'Blin', 'Guillon', 'Mary', 'Coste', 'Pruvost',
        'Maury', 'Allain', 'Valentin', 'Godard', 'Joseph', 'Brunel', 'Marion', 'Texier', 'Seguin', 'Raynaud', 'Bourdon',
        'Raymond', 'Bonneau', 'Chauvet', 'Maurice', 'Legendre', 'Loiseau', 'Ferrand', 'Toussaint', 'Techer', 'Lombard',
        'Lefort', 'Couturier', 'Bousquet', 'Diaz', 'Riou', 'Clerc', 'Weiss', 'Imbert', 'Jourdan', 'Delahaye', 'Gilles',
        'Guibert', 'Begue', 'Descamps', 'Delmas', 'Peltier', 'Dupre', 'Chartier', 'Martineau', 'Laroche', 'Leconte',
        'Maillet', 'Parent', 'Labbe', 'Potier', 'Bazin', 'Normand', 'Pottier', 'Torres', 'Lagarde', 'Blot', 'Jacquot',
        'Lemonnier', 'Grenier', 'Rocher', 'Bonnin', 'Boutin', 'Fischer', 'Munoz', 'Neveu', 'Lacombe', 'Mendes',
        'Delannoy', 'Auger', 'Wagner', 'Fouquet', 'Mace', 'Ramos', 'Pages', 'Petitjean', 'Chauveau', 'Foucher', 'Peron',
        'Guyon', 'Gallet', 'Rousset', 'Traore', 'Bernier', 'Vallet', 'Letellier', 'Bouvet', 'Hamel', 'Chretien',
        'Faivre', 'Boulay', 'Thierry', 'Samson', 'Ledoux', 'Salmon', 'Gosselin', 'Lecoq', 'Pires', 'Leleu', 'Becker',
        'Diallo', 'Merle', 'Valette',
    )

    prefixes = ('de', 'de la', 'Le', 'du')



# coding=utf-8
from __future__ import unicode_literals
from .. import Provider as BaseProvider


class Provider(BaseProvider):
    """
    Source: http://planet-beruf.de/schuelerinnen/mein-beruf/berufe-von-a-z/
    """
    jobs = [
        'Altenpfleger',
        'Asphaltbauer',
        'Artist',
        'Augenoptiker',
        'Ausbaufacharbeiter',
        'Bäcker',
        'Bankkaufmann',
        'Beamter',
        'Binnenschiffer',
        'Chemikant',
        'Chirurgiemechaniker',
        'Designer',
        'Drogist',
        'Erzieher',
        'Elektroniker',
        'Fachinformatiker',
        'Fleischer',
        'Florist',
        'Forstwirst',
        'Friseur',
        'Informatiker',
        'Programmierer',
        'Techniker',
        'Mathematiker',
        'Koreanistiker',
        'Mechaniker',
        'Pfleger',
        'Polizist',
        'Pilot',
        'Arzt',
        'Krankenschwester',
        'Medizininformatiker',
        'Schornsteinfeger',
        'Winzer',
    ]



# coding=utf-8
from __future__ import unicode_literals
from collections import OrderedDict
from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    """
    Provider for company names for en_PH locale

    Company naming scheme and probabilities are inspired by and/or based on existing companies in the Philippines.

    Sources:
    - https://en.wikipedia.org/wiki/List_of_companies_of_the_Philippines
    - https://www.pse.com.ph/stockMarket/listedCompanyDirectory.html
    """

    formats = OrderedDict([
        ('{{random_company_adjective}} {{random_company_noun_chain}} {{company_type}} {{company_suffix}}', 0.24),
        ('{{random_company_acronym}} {{random_company_noun_chain}} {{company_type}} {{company_suffix}}', 0.24),
        ('{{last_name}} {{random_company_noun_chain}} {{company_type}} {{company_suffix}}', 0.16),
        ('{{random_company_adjective}} {{company_type}} {{company_suffix}}', 0.12),
        ('{{random_company_acronym}} {{company_type}} {{company_suffix}}', 0.12),
        ('{{last_name}} {{company_type}} {{company_suffix}}', 0.09),
        ('National {{random_company_product}} Corporation of the Philippines', 0.03),
    ])
    company_suffixes = OrderedDict([
        ('Inc.', 0.45),
        ('Corporation', 0.45),
        ('Limited', 0.1),
    ])
    company_types = (
        'Bank',
        'Banking',
        'Capital',
        'Company',
        'Construction',
        'Development',
        'Enterprise',
        'Equities',
        'Finance',
        'Foods',
        'Group',
        'Holdings',
        'Hotel',
        'Manufacturing',
        'Mining',
        'Properties',
        'Resorts',
        'Resources',
        'Services',
        'Shipping',
        'Solutions',
        'Technologies',
        'Trust',
        'Ventures',
    )
    company_products = (
        'Bottle',
        'Coconut',
        'Computer',
        'Electricity',
        'Flour',
        'Furniture',
        'Glass',
        'Newspaper',
        'Pillow',
        'Water',
    )
    company_nouns = (
        'Century',
        'City',
        'Crown',
        'Dragon',
        'Empire',
        'Genesis',
        'Gold',
        'King',
        'Liberty',
        'Millennium',
        'Morning',
        'Silver',
        'Star',
        'State',
        'Summit',
        'Sun',
        'Union',
        'World',
    )
    company_adjectives = (
        'Advanced',
        'Rising',
        'Double',
        'Triple',
        'Quad',
        'Allied',
        'Cyber',
        'Sovereign',
        'Great',
        'Far',
        'Northern',
        'Southern',
        'Eastern',
        'Western',
        'First',
        'Filipino',
        'Grand',
        'Manila',
        'Mega',
        'Metro',
        'Global',
        'Pacific',
        'Oriental',
        'Philippine',
        'Prime',
    )

    def company_type(self):
        return self.random_element(self.company_types)

    def random_company_adjective(self):
        return self.random_element(self.company_adjectives)

    def random_company_noun_chain(self):
        return ' '.join(self.random_elements(self.company_nouns, length=self.random_int(1, 2), unique=True))

    def random_company_product(self):
        return self.random_element(self.company_products)

    def random_company_acronym(self):
        letters = self.random_letters(self.random_int(2, 4))
        return ''.join(letters).upper()



from __future__ import unicode_literals
from .. import Provider as PhoneNumberProvider


class Provider(PhoneNumberProvider):
    # Source:
    # https://en.wikipedia.org/wiki/Telephone_numbers_in_the_State_of_Palestine

    cellphone_formats = (
        '{{country_code}} {{provider_code}} ### ####',
        '{{country_code}}{{provider_code}}#######',
        '0{{provider_code}} ### ####',
        '0{{provider_code}}#######',
    )

    telephone_formats = (
        '{{country_code}} 4 24# ####',
        '{{country_code}}424#####',
        '04 24# ####',
        '0424#####',

        '{{country_code}} 9 25# ####',
        '{{country_code}}925#####',
        '09 25# ####',
        '0925#####',

        '{{country_code}} 4 26# ####',
        '{{country_code}}426#####',
        '04 26# ####',
        '0426#####',

        '{{country_code}} 4 23# ####',
        '{{country_code}}423#####',
        '04 23# ####',
        '0423#####',

        '{{country_code}} 4 29# ####',
        '{{country_code}}429#####',
        '04 29# ####',
        '0429#####',

        '{{country_code}} 2 29# ####',
        '{{country_code}}229#####',
        '02 29# ####',
        '0229#####',

        '{{country_code}} 2 23# ####',
        '{{country_code}}223#####',
        '02 23# ####',
        '0223#####',

        '{{country_code}} 2 22# ####',
        '{{country_code}}222#####',
        '02 22# ####',
        '0222#####',

        '{{country_code}} 2 27# ####',
        '{{country_code}}227#####',
        '02 27# ####',
        '0227#####',

        '{{country_code}} 8 20# ####',
        '{{country_code}}820#####',
        '08 20# ####',
        '0820#####',

        '{{country_code}} 8 21# ####',
        '{{country_code}}821#####',
        '08 21# ####',
        '0821#####',

        '{{country_code}} 8 24# ####',
        '{{country_code}}824#####',
        '08 24# ####',
        '0824#####',

        '{{country_code}} 8 25# ####',
        '{{country_code}}825#####',
        '08 25# ####',
        '0825#####',

        '{{country_code}} 8 26# ####',
        '{{country_code}}826#####',
        '08 26# ####',
        '0826#####',

        '{{country_code}} 8 28# ####',
        '{{country_code}}828#####',
        '08 28# ####',
        '0828#####',

    )

    toll_foramts = (
        '1 700 ### ###',
        '1-700-###-###',
        '1 800 ### ###',
        '1-800-###-###',
    )

    services_phones_formats = (
        '100',
        '101',
        '102',
    )

    formats = cellphone_formats + \
        telephone_formats + \
        services_phones_formats + \
        toll_foramts

    def provider_code(self):
        return self.random_element([
            '59',
            '56',
        ])

    def country_code(self):
        return self.random_element([
            '00972',
            '+972',
            '00970',
            '+970',
        ])

    def cellphone_number(self):
        pattern = self.random_element(self.cellphone_formats)
        return self.numerify(self.generator.parse(pattern))

    def telephone_number(self):
        pattern = self.random_element(self.telephone_formats)
        return self.numerify(self.generator.parse(pattern))

    def service_phone_number(self):
        pattern = self.random_element(self.services_phones_formats)
        return self.numerify(self.generator.parse(pattern))

    def toll_number(self):
        pattern = self.random_element(self.toll_foramts)
        return self.numerify(self.generator.parse(pattern))

    def phone_number(self):
        pattern = self.random_element(self.formats)
        return self.numerify(self.generator.parse(pattern))



# coding=utf-8

from __future__ import unicode_literals

import re
from six import string_types

from .. import BaseProvider


class Provider(BaseProvider):

    upc_e_base_pattern = re.compile(r'^\d{6}$')
    upc_ae_pattern1 = re.compile(
        r'^(?P<number_system_digit>[01])'   # The first digit must be 0 or 1
        r'(?=\d{11}$)'                      # followed by 11 digits of which
        r'(?P<mfr_code>\d{2})'              # the first 2 digits make up the manufacturer code,
        r'(?:(?P<extra>[012])0{4})'         # if immediately followed by 00000, 10000, or 20000,
        r'(?P<product_code>\d{3})'          # a 3-digit product code,
        r'(?P<check_digit>\d)$',            # and finally a check digit.
    )
    upc_ae_pattern2 = re.compile(
        r'^(?P<number_system_digit>[01])'   # The first digit must be 0 or 1
        r'(?=\d{11}$)'                      # followed by 11 digits of which
        r'(?P<mfr_code>\d{3,4}?)'           # the first 3 or 4 digits make up the manufacturer code,
        r'(?:0{5})'                         # if immediately followed by 00000,
        r'(?P<product_code>\d{1,2})'        # a 2-digit or single digit product code,
        r'(?P<check_digit>\d)$',            # and finally a check digit.
    )
    upc_ae_pattern3 = re.compile(
        r'^(?P<number_system_digit>[01])'   # The first digit must be 0 or 1
        r'(?=\d{11}$)'                      # followed by 11 digits of which
        r'(?P<mfr_code>\d{5})'              # the first 5 digits make up the manufacturer code,
        r'(?:0{4}(?P<extra>[5-9]))'         # if immediately followed by 0000 and a 5, 6, 7, 8, or 9,
        r'(?P<check_digit>\d)$',            # and finally a check digit.
    )

    def _ean(self, length=13, leading_zero=None):
        if length not in (8, 13):
            raise AssertionError("length can only be 8 or 13")

        code = [self.random_digit() for _ in range(length - 1)]
        if leading_zero is True:
            code[0] = 0
        elif leading_zero is False:
            code[0] = self.random_int(1, 9)

        if length == 8:
            weights = [3, 1, 3, 1, 3, 1, 3]
        elif length == 13:
            weights = [1, 3, 1, 3, 1, 3, 1, 3, 1, 3, 1, 3]

        weighted_sum = sum(x * y for x, y in zip(code, weights))
        check_digit = (10 - weighted_sum % 10) % 10
        code.append(check_digit)

        return ''.join(str(x) for x in code)

    def _convert_upc_a2e(self, upc_a):
        """
        Converts a 12-digit UPC-A barcode to its 8-digit UPC-E equivalent

        Not all UPC-A barcodes can be converted.

        :param upc_a: UPC-A barcode to convert
        :return: UPC-E equivalent barcode
        """
        if not isinstance(upc_a, string_types):
            raise TypeError('`upc_a` is not a string')
        m1 = self.upc_ae_pattern1.match(upc_a)
        m2 = self.upc_ae_pattern2.match(upc_a)
        m3 = self.upc_ae_pattern3.match(upc_a)
        if not any([m1, m2, m3]):
            raise ValueError('`upc_a` has an invalid value')
        upc_e_template = '{number_system_digit}{mfr_code}{product_code}{extra}{check_digit}'
        if m1:
            upc_e = upc_e_template.format(**m1.groupdict())
        elif m2:
            groupdict = m2.groupdict()
            groupdict['extra'] = str(len(groupdict.get('mfr_code')))
            upc_e = upc_e_template.format(**groupdict)
        else:
            groupdict = m3.groupdict()
            groupdict['product_code'] = ''
            upc_e = upc_e_template.format(**groupdict)
        return upc_e

    def _upc_ae(self, base=None, number_system_digit=None):
        """
        Creates a 12-digit UPC-A barcode that can be converted to UPC-E

        Notes on this method:
        - Please view notes on `upc_a()` and `upc_e()` first.

        :param base: A 6-digit string
        :param number_system_digit: 0 or 1
        :return: 12-digit UPC-A barcode that can be converted to UPC-E
        """
        if isinstance(base, string_types) and self.upc_e_base_pattern.match(base):
            base = [int(x) for x in base]
        else:
            base = [self.random_int(0, 9) for _ in range(6)]
        if number_system_digit not in [0, 1]:
            number_system_digit = self.random_int(0, 1)

        if base[-1] <= 2:
            code = base[:2] + base[-1:] + [0] * 4 + base[2:-1]
        elif base[-1] <= 4:
            code = base[:base[-1]] + [0] * 5 + base[base[-1]:-1]
        else:
            code = base[:5] + [0] * 4 + base[-1:]

        code.insert(0, number_system_digit)
        weights = [3, 1, 3, 1, 3, 1, 3, 1, 3, 1, 3]
        weighted_sum = sum(x * y for x, y in zip(code, weights))
        check_digit = (10 - weighted_sum % 10) % 10
        code.append(check_digit)
        return ''.join(str(x) for x in code)

    def ean(self, length=13):
        return self._ean(length)

    def ean8(self):
        return self._ean(8)

    def ean13(self, leading_zero=None):
        """
        Creates an EAN-13 barcode

        :param leading_zero: Leading digit will be 0 if True, 1-9 if False, and 0-9 otherwise
        :return: An EAN-13 barcode
        """
        return self._ean(13, leading_zero=leading_zero)

    def upc_a(self, upc_ae_mode=False, base=None, number_system_digit=None):
        """
        Creates a 12-digit UPC-A barcode

        Notes on UPC-A barcode and this method:
        - EAN-13 is a superset of UPC-A. Simply put, leading zero + UPC-A = EAN-13.
        - For the lack of a concise and better term, this provider uses the term "upc_ae" to mean
          that a UPC-A barcode can be converted to UPC-E.
        - If `upc_ae_mode` is enabled, this method will only return UPC-A barcodes that can be
          converted to UPC-E. In this mode the leading digit (number system) of the UPC-A barcode
          may only start with 0 or 1.
        - When `upc_ae_mode` is enabled, the `number_system_digit` may be explicitly set to 0 or 1.
          If it is not set or is invalid, either values will be chosen at random.
        - When `upc_ae_mode` is enabled, a 6-digit UPC-E string `base` may be explicitly set. If it
          is not set or is invalid, a random 6 digit combination will be used.
        - If `upc_ae_mode` is disabled, the values of `base` and `number_system_digit` are ignored.

        :param upc_ae_mode: Set to True explicitly to enable
        :param base: A 6-digit string
        :param number_system_digit: 0 or 1
        :return: 12-digit UPC-A barcode
        """
        if upc_ae_mode is True:
            return self._upc_ae(base=base, number_system_digit=number_system_digit)
        else:
            ean13 = self.ean13(leading_zero=True)
            return ean13[1:]

    def upc_e(self, base=None, number_system_digit=None, safe_mode=True):
        """
        Creates an 8-digit UPC-E barcode

        Notes on UPC-E barcode and this method:
        - UPC-E barcodes can be expressed in 6, 7, or 8-digit formats, but this method uses the
          8 digit format, since it is trivial to convert to the other two formats.
        - The first digit of the barcode denotes the number system used, either 0 or 1.
        - The last digit is the check digit (more on that below).
        - The remaining 6 digits are a bit more involved, but they are referred to as the `base`
          argument elsewhere in this provider (for the lack of a concise and better term).
        - Each UPC-E `base` have 2 UPC-A equivalents depending on the number system used.
        - If the value of `number_system_digit` is not 0 or 1, either values will be chosen at random.
        - A 6-digit string `base` may also be explicitly set. If invalid, it will be ignored, and a
          new value for `base` will be generated.
        - Internally, this method first generates a UPC-A barcode that can be converted to UPC-E, and
          then actually performs a conversion for the result. This is because both the number
          system and check digits of the UPC-E barcode are inherited from its UPC-A equivalent.
        - Then there is `safe_mode`. Enabling this guarantees that every UPC-E barcode generated can
          be converted to UPC-A, and that UPC-A value can be converted back again to UPC-E and still
          be equal to the original value. In other words, the statement a2e(e2a(e)) == e holds True.
        - As to why there is `safe_mode`, it is because there are some UPC-E values that share the
          same UPC-A equivalent. For example, any UPC-E barcode of the form abc0000d, abc0003d, and
          abc0004d share the same UPC-A value abc00000000d, but that UPC-A value will only convert
          to abc0000d because of (a) how UPC-E is just a zero-suppressed version of UPC-A and
          (b) the rules around the conversion.

        :param base: A 6-digit string
        :param number_system_digit: First digit of the barcode
        :param safe_mode: True or False. Guarantees that every UPC-E barcode generated can be
                          converted back-and-forth between UPC-A and UPC-E.
        :return: 8-digit UPC-E barcode
        """
        if safe_mode is not False:
            upc_ae = self._upc_ae(base=base, number_system_digit=number_system_digit)
            return self._convert_upc_a2e(upc_ae)
        else:
            upc_ae = self._upc_ae(base=base, number_system_digit=number_system_digit)
            return upc_ae[0] + ''.join(str(x) for x in base) + upc_ae[-1]



# coding=utf-8

import re
import unicodedata

import six

_re_pattern = re.compile(r'[^\w\s-]', flags=re.U)
_re_pattern_allow_dots = re.compile(r'[^\.\w\s-]', flags=re.U)
_re_spaces = re.compile(r'[-\s]+', flags=re.U)


def slugify(value, allow_dots=False, allow_unicode=False):
    """
    Converts to lowercase, removes non-word characters (alphanumerics and
    underscores) and converts spaces to hyphens. Also strips leading and
    trailing whitespace. Modified to optionally allow dots.

    Adapted from Django 1.9
    """
    if allow_dots:
        pattern = _re_pattern_allow_dots
    else:
        pattern = _re_pattern

    value = six.text_type(value)
    if allow_unicode:
        value = unicodedata.normalize('NFKC', value)
        value = pattern.sub('', value).strip().lower()
        return _re_spaces.sub('-', value)
    value = unicodedata.normalize('NFKD', value).encode(
        'ascii', 'ignore').decode('ascii')
    value = pattern.sub('', value).strip().lower()
    return _re_spaces.sub('-', value)



# coding=utf-8

from __future__ import unicode_literals

from faker.utils.datetime_safe import datetime
from .. import Provider as CompanyProvider


def calculate_checksum(value):
    factors = [3, 7, 2, 4, 10, 3, 5, 9, 4, 6, 8][-len(value):]
    check_sum = 0
    for number, factor in zip(value, factors):
        check_sum += int(number) * factor

    return str((check_sum % 11) % 10)


class Provider(CompanyProvider):
    formats = (
        '{{company_prefix}} «{{last_name}}»',
        '{{company_prefix}} «{{last_name}} {{last_name}}»',
        '{{company_prefix}} «{{last_name}}-{{last_name}}»',
        '{{company_prefix}} «{{last_name}}, {{last_name}} и {{last_name}}»',
        '{{last_name}}',
    )

    company_prefixes = (
        'РАО', 'АО', 'ИП', 'НПО',
    )

    def company_prefix(self):
        return self.random_element(self.company_prefixes)

    def businesses_inn(self):
        """
        Returns tax identification number for businesses (ru. идентификационный номер налогоплательщика, ИНН).
        """
        region = '%02d' % self.random_int(min=1, max=92)
        inspection = '%02d' % self.random_int(min=1, max=99)
        tail = '%05d' % self.random_int(min=1, max=99999)
        result = region + inspection + tail

        return result + calculate_checksum(result)

    def individuals_inn(self):
        """
        Returns tax identification number for individuals (ru. идентификационный номер налогоплательщика, ИНН).
        """
        region = '%02d' % self.random_int(min=1, max=92)
        inspection = '%02d' % self.random_int(min=1, max=99)
        tail = '%06d' % self.random_int(min=1, max=999999)
        result = region + inspection + tail
        result += calculate_checksum(result)

        return result + calculate_checksum(result)

    def businesses_ogrn(self):
        """
        Returns primary state registration number for businesses
        (ru. основной государственный регистрационный номер, ОГРН).
        """
        sign = self.random_element(('1', '5'))
        year = '%02d' % self.random_int(min=1, max=datetime.now().year - 2000)
        region = '%02d' % self.random_int(min=1, max=92)
        tail = '%07d' % self.random_int(min=1, max=9999999)

        result = sign + year + region + tail

        return result + str((int(result) % 11) % 10)

    def individuals_ogrn(self):
        """
        Returns primary state registration number for individuals
        (ru. основной государственный регистрационный номер, ОГРН).
        """
        year = '%02d' % self.random_int(min=1, max=datetime.now().year - 2000)
        region = '%02d' % self.random_int(min=1, max=92)
        tail = '%09d' % self.random_int(min=1, max=999999999)

        result = '3' + year + region + tail

        return result + str((int(result) % 13) % 10)

    def kpp(self):
        """
        Returns tax registration reason code (ru. код причины постановки на учет, КПП).
        """
        region = '%02d' % self.random_int(min=1, max=92)
        inspection = '%02d' % self.random_int(min=1, max=99)
        reason = self.random_element(('01', '43', '44', '45'))
        tail = '%03d' % self.random_int(min=1, max=999)

        return region + inspection + reason + tail



# coding=utf-8
from __future__ import unicode_literals
from .. import Provider as DateTimeProvider


class Provider(DateTimeProvider):
    """Provider for datetimes for fil_PH locale"""

    DAY_NAMES = {
        '0': 'Linggo',
        '1': 'Lunes',
        '2': 'Martes',
        '3': 'Miyerkules',
        '4': 'Huwebes',
        '5': 'Biyernes',
        '6': 'Sabado',
    }
    MONTH_NAMES = {
        '01': 'Enero',
        '02': 'Pebrero',
        '03': 'Marso',
        '04': 'Abril',
        '05': 'Mayo',
        '06': 'Hunyo',
        '07': 'Hulyo',
        '08': 'Agosto',
        '09': 'Setyembre',
        '10': 'Oktubre',
        '11': 'Nobyembre',
        '12': 'Disyembre',
    }

    def day_of_week(self):
        day = self.date('%w')
        return self.DAY_NAMES[day]

    def month_name(self):
        month = self.month()
        return self.MONTH_NAMES[month]



# coding=utf-8
from __future__ import unicode_literals
from collections import OrderedDict

from .. import BaseProvider

localized = True


class CreditCard(object):

    def __init__(
            self,
            name,
            prefixes,
            length=16,
            security_code='CVC',
            security_code_length=3):
        self.name = name
        self.prefixes = prefixes
        self.length = length
        self.security_code = security_code
        self.security_code_length = security_code_length


class Provider(BaseProvider):

    # Prefixes from:
    # * https://en.wikipedia.org/wiki/Payment_card_number#Issuer_identification_number_.28IIN.29
    # * https://www.regular-expressions.info/creditcard.html
    # * https://creditcardjs.com/credit-card-type-detection
    prefix_maestro = ['5018', '5020', '5038', '56##', '57##', '58##',
                      '6304', '6759', '6761', '6762', '6763', '0604', '6390']
    prefix_mastercard = ['51', '52', '53', '54', '55', '222%', '223', '224',
                         '225', '226', '227', '228', '229', '23', '24', '25',
                         '26', '270', '271', '2720']
    prefix_visa = ['4']
    prefix_amex = ['34', '37']
    prefix_discover = ['6011', '65']
    prefix_diners = ['300', '301', '302', '303', '304', '305', '36', '38']
    prefix_jcb16 = ['35']
    prefix_jcb15 = ['2131', '1800']

    credit_card_types = OrderedDict((
        ('maestro', CreditCard('Maestro',
                                    prefix_maestro, 12, security_code='CVV')),
        ('mastercard', CreditCard('Mastercard',
                                  prefix_mastercard, 16, security_code='CVV')),
        ('visa16', CreditCard('VISA 16 digit', prefix_visa)),
        ('visa13', CreditCard('VISA 13 digit', prefix_visa, 13)),
        ('visa19', CreditCard('VISA 19 digit', prefix_visa, 19)),
        ('amex', CreditCard('American Express', prefix_amex,
                            15, security_code='CID', security_code_length=4)),
        ('discover', CreditCard('Discover', prefix_discover)),
        ('diners', CreditCard('Diners Club / Carte Blanche', prefix_diners, 14)),
        ('jcb15', CreditCard('JCB 15 digit', prefix_jcb15, 15)),
        ('jcb16', CreditCard('JCB 16 digit', prefix_jcb16)),
    ))
    credit_card_types['visa'] = credit_card_types['visa16']
    credit_card_types['jcb'] = credit_card_types['jcb16']

    luhn_lookup = {'0': 0, '1': 2, '2': 4, '3': 6, '4': 8,
                   '5': 1, '6': 3, '7': 5, '8': 7, '9': 9}

    def credit_card_provider(self, card_type=None):
        """ Returns the provider's name of the credit card. """
        if card_type is None:
            card_type = self.random_element(self.credit_card_types.keys())
        return self._credit_card_type(card_type).name

    def credit_card_number(self, card_type=None):
        """ Returns a valid credit card number. """
        card = self._credit_card_type(card_type)
        prefix = self.random_element(card.prefixes)
        number = self._generate_number(self.numerify(prefix), card.length)
        return number

    def credit_card_expire(self, start='now', end='+10y', date_format='%m/%y'):
        expire_date = self.generator.date_time_between(start, end)
        return expire_date.strftime(date_format)

    def credit_card_full(self, card_type=None):
        card = self._credit_card_type(card_type)

        tpl = ('{provider}\n'
               '{owner}\n'
               '{number} {expire_date}\n'
               '{security}: {security_nb}\n')

        tpl = tpl.format(provider=card.name,
                         owner=self.generator.parse(
                             "{{first_name}} {{last_name}}"),
                         number=self.credit_card_number(card),
                         expire_date=self.credit_card_expire(),
                         security=card.security_code,
                         security_nb=self.credit_card_security_code(card))

        return self.generator.parse(tpl)

    def credit_card_security_code(self, card_type=None):
        """ Returns a security code string. """
        sec_len = self._credit_card_type(card_type).security_code_length
        return self.numerify('#' * sec_len)

    def _credit_card_type(self, card_type=None):
        """ Returns a random credit card type instance. """
        if card_type is None:
            card_type = self.random_element(self.credit_card_types.keys())
        elif isinstance(card_type, CreditCard):
            return card_type
        return self.credit_card_types[card_type]

    def _generate_number(self, prefix, length):
        """
        'prefix' is the start of the CC number as a string, any number of digits.
        'length' is the length of the CC number to generate. Typically 13 or 16
        """
        number = prefix
        # Generate random char digits
        number += '#' * (length - len(prefix) - 1)
        number = self.numerify(number)
        reverse = number[::-1]
        # Calculate sum
        tot = 0
        pos = 0
        while pos < length - 1:
            tot += Provider.luhn_lookup[reverse[pos]]
            if pos != (length - 2):
                tot += int(reverse[pos + 1])
            pos += 2
        # Calculate check digit
        check_digit = (10 - (tot % 10)) % 10
        number += str(check_digit)
        return number



# coding=utf-8
from __future__ import unicode_literals
from .. import Provider as BaseProvider


class Provider(BaseProvider):
    jobs = [
        'Αγγειοπλάστης',
        'Αγιογράφος Βυζαντινών Εικόνων και Τοιχογραφιών',
        'Αγρονόμος Τοπογράφος Μηχανικός',
        'Αγρότης Καλλιεργητής',
        'Αεροναυπηγός Μηχανικός',
        'Αεροσυνοδός',
        'Αθλητικός Δημοσιογράφος – Ρεπόρτερ',
        'Αισθητικός - Ειδικός Εφαρμογών Αισθητικής',
        'Αισθητικός Αρωματοθεραπείας',
        'Αισθητικός Επαγγελματικού Μακιγιάζ Θεάτρου και Σκηνής',
        'Αισθητικός Ποδολογίας και Καλλωπισμού Νυχιών',
        'Τεχνίτης Περιποίησης Νυχιών',
        'Ακροβάτης',
        'Ακτινοδιαγνώστης',
        'Αλουμινάς',
        'Αναισθησιολόγος',
        'Αναλογιστής',
        'Ανθοκόμος',
        'Αξιωματικός Μηχανικός Ναυσιπλοΐας',
        'Αξιωματικός Νοσηλευτικής',
        'Αξιωματικός Πολεμικής Αεροπορίας',
        'Αξιωματικός Στρατού Ξηράς',
        'Αξιωματικός Σωμάτων Ενόπλων Δυνάμεων',
        'Αξιωματικός Φαρμακοποιός',
        'Αργυροχρυσοχόος',
        'Αρτοποιός',
        'Αρχαιολόγος',
        'Αρχειοθέτης',
        'Αρχιμάγειρας',
        'Αρχισυντάκτης',
        'Αρχιτέκτονας',
        'Αστυνομικός',
        'Ασφαλιστής',
        'Βιβλιοδέτης',
        'Βιβλιοθηκονόμος',
        'Βιολόγος',
        'Βιοπληροφορικός',
        'Βοηθός Αναισθησιολόγου',
        'Βοηθός Αρτοποιίας – Ζαχαροπλαστικής',
        'Βοηθός Ιατρικών και Βιολογικών Εργαστηρίων, Βιοχημικού και Μικροβιολογικού Εργαστηρίου',
        'Βοηθός Ιατρικών και Βιολογικών Εργαστηρίων, Αιματολογικού Εργαστηρίου',
        'Βοηθός Κομμωτή',
        'Βοηθός Λογιστή',
        'Βοηθός Μαγειρικής Τέχνης',
        'Βοηθός Νοσηλευτή Μονάδας Εντατικής Θεραπείας',
        'Βοηθός Νοσηλευτή Τραυματολογίας',
        'Βοηθός Νοσηλευτή Χειρουργείου',
        'Βοηθός Οδοντιάτρου',
        'Βοηθός Φαρμακοποιού',
        'Βοηθός Φυσικοθεραπευτή σε Ιαματικές Πηγές και Λουτρά',
        'Βοηθός Φυσικοθεραπευτή - Υπάλληλος Εργαστηρίου Φυσικοθεραπείας',
        'Βοτανολόγος',
        'Βρεφοκόμος',
        'Γεωγράφος',
        'Γεωλόγος',
        'Γεωπόνος Φυτικής Παραγωγής',
        'Γεωργός Βιολογικής Γεωργίας',
        'Γεωτεχνολόγος',
        'Γεωτρυπανιστής',
        'Γεωφυσικός',
        'Γλωσσολόγος',
        'Γλύπτης',
        'Γουνοποιός',
        'Γραμματέας',
        'Γραμματέας Νομικών Επαγγελμάτων',
        'Γραφίστας',
        'Γραφολόγος',
        'Δάσκαλος',
        'Δασολόγος',
        'Δασοφύλακας - Ειδικός Δασικής Προστασίας',
        'Δημοσιογράφος Συντάκτης - Ρεπόρτερ Δημοσιογραφίας',
        'Διαδικτυακός Διαφημιστής',
        'Διαιτητής Αθλήματος',
        'Διαιτολόγος - Ειδικός Εφαρμογών Διαιτητικής',
        'Διακοσμητής - Τεχνικός Διακόσμησης',
        'Διαφημιστής',
        'Διαχειριστής Logistics',
        'Διαχειριστής Τεχνολογίας Internet',
        'Διερμηνέας',
        'Διευθυντής Eπιχειρήσεων',
        'Διευθύνων Επιχειρηματίας',
        'Δικαστής',
        'Δικαστικός Επιμελητής',
        'Δικηγόρος',
        'Διοικητικό Στέλεχος Αθλητισμού',
        'Διοικητικό και Οικονομικό Στέλεχος Επιχειρήσεων',
        'Διοικητικός Υπάλληλος',
        'Διπλωμάτης',
        'Εγκληματολόγος',
        'Εθνολόγος - Λαογράφος',
        'Ειδικός 3D Animation',
        'Ειδικός Αεροπορικών Εταιριών',
        'Ειδικός Αργυροχρυσοχοΐας',
        'Ειδικός Ασφαλιστικών Εργασιών',
        'Ειδικός Γεωγραφικών Πληροφοριακών Συστημάτων',
        'Ειδικός Διοίκησης Πολιτισμικών Μονάδων',
        'Ειδικός Εκπαίδευσης Ενηλίκων',
        'Ειδικός Εμπορίας, Διαφήμισης και Προώθησης Προϊόντων',
        'Ειδικός Εμπορικών Επιχειρήσεων',
        'Ειδικός Ενδυματολογίας και Σχεδίασης Μόδας',
        'Ειδικός Επιστήμων Θάλασσας',
        'Ειδικός Ηλεκτρονικής Σχεδίασης Εντύπου',
        'Ειδικός Ιατρικών Συσκευών Ακτινοθεραπείας',
        'Ειδικός Ιατρικών Συσκευών Απεικονίσεων',
        'Ειδικός Ιατρικών και Βιολογικών Εργαστηρίων Αιματολογικού Εργαστηρίου',
        'Ειδικός Κινηματογράφου',
        'Ειδικός Μάρκετινγκ',
        'Ειδικός Μουσικής με Τεχνολογία Η/Υ',
        'Ειδικός Ναυτιλιακής Κατεύθυνσης',
        'Ειδικός Νοσηλευτικής Ατόμων με Ψυχικές Παθήσεις',
        'Ειδικός Νοσηλευτικής Μονάδων Εντατικής Θεραπείας',
        'Ειδικός Νοσηλευτικής Ογκολογικών Παθήσεων',
        'Ειδικός Νοσηλευτικής Τραυματολογίας',
        'Ειδικός Ξενοδοχειακής Ψυχαγωγίας',
        'Ειδικός Ξενοδοχειακών Υπηρεσιών και Τροφοδοσίας',
        'Ειδικός Οδοντοτεχνικής Ορθοδοντικής',
        'Ειδικός Οδοντοτεχνικής Πορσελάνης',
        'Ειδικός Οπτικών και Ηχητικών Εφέ',
        'Ειδικός Παθολόγος Λόγου και Ομιλίας',
        'Ειδικός Πληροφοριακών Συστημάτων με Πολυμέσα',
        'Ειδικός Προσχολικής Αγωγής Δραστηριοτήτων Δημιουργίας και Έκφρασης',
        'Ειδικός Προσχολικής Αγωγής Ημερήσιας Φροντίδας Παιδιών με Ειδικές Ανάγκες',
        'Ειδικός Πρόληψης και Θεραπείας Εξαρτημένων Ατόμων',
        'Ειδικός Πωλήσεων',
        'Ειδικός Στατιστικής Έρευνας Αγοράς και Δημοσκοπήσεων – Στέλεχος Στατιστικής Δημοσκόπησης',
        'Ειδικός Συναρμολόγησης Εικόνας (Μοντέρ) - Ηλεκτρονική Επεξεργασία Εικόνας',
        'Ειδικός Σχεδίασης Διαδραστικών και Βιομηχανικών Προϊόντων και Συστημάτων',
        'Ειδικός Σχεδίου και Μάρκετινγκ Μόδας',
        'Ειδικός Σχεδιασμού Αστικής και Περιφερειακής Ανάπτυξης',
        'Ειδικός Σχεδιασμού Επίπλου',
        'Ειδικός Τροφοδοσίας-Σίτισης',
        'Υδροθεραπευτής - Λουτροθεραπευτής',
        'Ειδικός Φοροτεχνικού Γραφείου',
        'Ειδικός Φρουρός',
        'Ειδικός Ψηφιακών Μορφών Τέχνης',
        'Εικονογράφος - Σκιτσογράφος',
        'Εικονολήπτης',
        'Εισαγγελέας',
        'Εκδότης Βιβλίων',
        'Εκκλησιαστική και Πολιτιστική Κατάρτιση',
        'Εκπαιδευτής Ζώων',
        'Εκπαιδευτής Υποψήφιων Οδηγών Αυτοκινήτων και Μοτοσικλετών',
        'Εκπαιδευτικός Ειδικής Αγωγής',
        'Εκτελωνιστής και Πράκτορας Μεταφορών',
        'Εκτιμητής Έργων Τέχνης',
        'Εκτιμητής και Εκπλειστηριαστής',
        'Εκφωνητής',
        'Ελαιοχρωματιστής',
        'Ελεγκτής Εναέριας Κυκλοφορίας',
        'Ελεγκτής Ολικής Ποιότητας',
        'Ενδυματολόγος',
        'Επαγγελματίας Αθλητής',
        'Επαγγελματίας Δύτης',
        'Επαγγελματικό Μακιγιάζ',
        'Επιθεωρητής κτιρίων και πυρασφάλειας',
        'Επικοινωνιολόγος',
        'Επιμελητής Πτήσεων',
        'Επιπλοποιός',
        'Επισκέπτης Υγείας',
        'Επισκευαστής Λαμαρινών Αυτοκινήτων-Οχημάτων',
        'Επισκευαστής Πλαστικών Σκαφών',
        'Επιστήμων Πληροφορικής και Η/Υ',
        'Επόπτης Δημόσιας Υγείας',
        'Εργοθεραπευτής',
        'Ζαχαροπλάστης',
        'Ζωγράφος',
        'Ζωολόγος',
        'Ηθοποιός',
        'Ηλεκτρολόγος',
        'Ηλεκτρολόγος Μηχανικός και Μηχανικός Η/Υ',
        'Ηλεκτρονικός',
        'Ηχολήπτης',
        'Θεατρολόγος',
        'Θεολόγος-Ιεροκήρυκας',
        'Ιατρικός Επισκέπτης',
        'Ιατροδικαστής',
        'Ιατρός',
        'Ιερέας',
        'Ιεροψάλτης',
        'Ιστορικός',
        'Ιστορικός Τέχνης',
        'Ιχθυοκαλλιεργητής',
        'Ιχθυοπαθολόγος',
        'Καθηγητής Γυμναστικής',
        'Καθηγητής Δευτεροβάθμιας Εκπαίδευσης',
        'Καθηγητής Πρωτοβάθμιας Εκπαίδευσης',
        'Καθηγητής Τριτοβάθμιας Εκπαίδευσης',
        'Καλλιτέχνης',
        'Καμαριέρης - Καμαριέρα',
        'Κασκαντέρ',
        'Κατασκευαστής Ειδών Ενδυμασίας',
        'Κατασκευαστής Υποδημάτων',
        'Κεραμίστας',
        'Κηπουρός - Ανθοκηπουρός - Κηποτεχνικός',
        'Κλειδαράς',
        'Κλιματολόγος',
        'Κλωστοϋφαντουργός',
        'Κλόουν - Γελοτοποιός',
        'Κοινωνικός Ανθρωπολόγος',
        'Κοινωνικός Λειτουργός',
        'Κοινωνιολόγος',
        'Κομμωτής – Τεχνικός Περιποίησης Κόμης',
        'Κορνιζοποιός',
        'Κοσμετολόγος',
        'Κρεοπώλης',
        'Κριτικός Τέχνης',
        'Κτηματομεσίτης',
        'Μεσίτης Αστικών Συμβάσεων',
        'Διαχειριστής Ακίνητης Περιουσίας',
        'Κτηνίατρος',
        'Κτηνοτρόφος',
        'Κόπτης Ενδυμάτων',
        'Κόφτης Υποδημάτων',
        'Λατόμος',
        'Λιθογράφος Ψηφιακής Τεχνολογίας',
        'Λιμενεργάτης',
        'Λιμενικός',
        'Λογιστής - Ειδικός Μηχανογραφημένου Λογιστηρίου',
        'Λογοθεραπευτής',
        'Λογοτέχνης',
        'Μαθηματικός',
        'Μαιευτής – Μαία',
        'Μακιγιέρ/Μακιγιέζ',
        'Μαρμαράς',
        'Μελισσοκόμος',
        'Μεσίτης Ναυτιλιακών Συμβάσεων',
        'Μεταφραστής',
        'Μετεωρολόγος',
        'Μηχανικός Αεροσκαφών',
        'Μηχανικός Αυτοκινήτων',
        'Μηχανικός Διαχείρισης Ενεργειακών Πόρων',
        'Μηχανικός Εμπορικού Ναυτικού',
        'Μηχανικός Επιστήμης και Τεχνολογίας των Υλικών',
        'Μηχανικός Η/Υ, Τηλεπικοινωνιών και Δικτύων',
        'Μηχανικός Μεταλλείων',
        'Μηχανικός Ορυκτών Πόρων',
        'Μηχανικός Παραγωγής και Διοίκησης',
        'Μηχανικός Περιβάλλοντος',
        'Μηχανικός Πληροφοριακών και Επικοινωνιακών Συστημάτων',
        'Μηχανικός Πλοίων',
        'Μηχανικός Πολεμικής Αεροπορίας',
        'Μηχανικός Πολεμικού Ναυτικού',
        'Μηχανικός Τηλεπικοινωνιών',
        'Μηχανικός Χωροταξίας και Περιφερειακής Ανάπτυξης',
        'Μηχανοδηγός Τρένου',
        'Μηχανολόγος Μηχανικός',
        'Μηχανολόγος Μηχανικός Βιομηχανίας',
        'Μηχανολόγος Μηχανικός Συστημάτων Αυτόματου Ελέγχου και Ρομποτικής',
        'Μηχανοσυνθέτης Αεροσκαφών',
        'Μικροβιολόγος',
        'Μουσειολόγος Μουσειογράφος και Σχεδιαστής Εκθέσεων',
        'Μουσικολόγος',
        'Μουσικός',
        'Μουσικός Ενορχηστρωτής',
        'Μουσικός Επιμελητής',
        'Μόνιμος Υπαξιωματικός Αεροπορίας',
        'Μόνιμος Υπαξιωματικός Ναυτικού',
        'Μόνιμος Υπαξιωματικός Στρατού Ξηράς',
        'Νανοτεχνολόγος',
        'Ναυαγοσώστης',
        'Ναυπηγός Μηχανικός',
        'Ναύτης',
        'Νηπιαγωγός',
        'Νοσοκόμα Παίδων',
        'Νοσοκόμος-Νοσηλευτής',
        'Ντετέκτιβ',
        'Ξεναγός',
        'Ξενοδοχοϋπάλληλος',
        'Ξυλουργός',
        'Οδηγός Αυτοκινήτου Αγώνων',
        'Οδηγός Οχημάτων Δημοσίας Χρήσης',
        'Οδηγός ΤΑΞΙ',
        'Οδοντίατρος',
        'Οδοντοτεχνίτης',
        'Οικονομολόγος',
        'Οινολόγος',
        'Οπτικός',
        'Οργανοποιός',
        'Οργανωτής Συνεδρίων, Εκθέσεων και Εκδηλώσεων',
        'Οργανωτικός Συντονιστής Παραγωγής - Παραγωγός',
        'Οφθαλμίατρος',
        'Παιδίατρος',
        'Παραγωγός Κινηματογράφου και Τηλεόρασης',
        'Παραγωγός Προϊόντων Αλευρόμυλων, Παραγωγή Αμύλων και Προϊόντων Αμύλου',
        'Παραγωγός Ραδιοφωνικών Εκπομπών',
        'Πατωματζής (Επαγγέλματα Οικοδομής)',
        'Περιβαλλοντολόγος',
        'Πιλοποιός',
        'Πιλότος',
        'Πλέκτης',
        'Πλακάς',
        'Πλανόδιος Πωλητής',
        'Πλοίαρχος',
        'Πλοηγός Σκαφών Αναψυχής',
        'Πολιτικός Επιστήμονας',
        'Πολιτικός Μηχανικός',
        'Πολιτικός Συντάκτης',
        'Προγραμματιστής Συστημάτων και Εφαρμογών Η/Υ',
        'Προπονητής Αθλήματος',
        'Προπονητής Αντισφαίρισης',
        'Προπονητής Γυμναστικής με Βάρη',
        'Προπονητής Ελεύθερης Γυμναστικής',
        'Προπονητής Καλαθοσφαίρισης',
        'Προπονητής Κλασικού Αθλητισμού',
        'Προπονητής Κολύμβησης',
        'Προπονητής Πετοσφαίρισης',
        'Προπονητής Ποδοσφαίρου',
        'Πυροσβέστης',
        'Πωλητής',
        'Πωλητής Οικολογικών Τροφίμων και άλλων Ειδών',
        'Ράφτης',
        'Ρεσεψιονίστ Ξενοδοχείου',
        'Στέλεχος Υποδοχής Ξενοδοχείου',
        'Σεισμολόγος',
        'Σεναριογράφος',
        'Σερβιτόρος',
        'Σερβιτόρος Ποτών - Μπάρμαν',
        'Σιδεράς - Σιδηρουργός',
        'Σιδερωτής',
        'Σκηνογράφος',
        'Σκηνοθέτης Τηλεόρασης',
        'Σοβατζής',
        'Στέλεχος Marketing',
        'Στέλεχος Διεθνούς Εμπορίου',
        'Στέλεχος Εκδοτικών Επιχειρήσεων',
        'Στέλεχος Κοστολόγησης στη Βιομηχανία - Βιοτεχνία',
        'Στέλεχος Μικρομεσαίων Επιχειρήσεων',
        'Στέλεχος Τουρισμού',
        'Στέλεχος Τραπεζικών Εργασιών',
        'Στέλεχος Υπηρεσιών Ασφαλείας',
        'Στέλεχος Υπηρεσιών Εφοδιαστικής Αλυσίδας',
        'Στέλεχος Χρηματιστηριακών Εργασιών',
        'Στατιστικολόγος',
        'Στρατιωτικός Ψυχολόγος',
        'Στυλίστας',
        'Συμβολαιογράφος',
        'Συνοδός Εδάφους',
        'Συντηρητής Αρχαιοτήτων και Έργων Τέχνης',
        'Τεχνικός Συντήρησης Έργων Τέχνης',
        'Σφουγγαράς',
        'Σχεδιαστής Επίπλου',
        'Σχεδιαστής Κινουμένων Σχεδίων',
        'Σχεδιαστής Μέσω Συστημάτων Η/Υ',
        'Σχεδιαστής Πολυμέσων - Ειδικός Πολυμέσων',
        'Σχεδιαστής Υποδημάτων',
        'Σχολικός Σύμβουλος',
        'Σύμβουλος Επιχειρήσεων',
        'Σύμβουλος Σχολικού και Επαγγελματικού Προσανατολισμού',
        'Ταμίας',
        'Ταξιδιωτικός Συνοδός και Συνοδός δρυμών',
        'Τουριστικός Συνοδός',
        'Ταπετσέρης',
        'Ταπητουργός',
        'Ταχυδρόμος',
        'Χειριστής Τηλεφωνικού Κέντρου',
        'Τεχνίτης Αδαμαντοκοπής Μαρμάρων',
        'Τεχνίτης Αλιείας - Ιχθυοπαραγωγής - Τεχνολόγος Υδατοκαλλιεργητής',
        'Τεχνίτης Δερμάτινων Ειδών',
        'Τεχνίτης Φαρμάκων - Καλλυντικών - Τεχνικός Φαρμάκων Καλλυντικών και Παρεμφερών Προϊόντων',
        'Τεχνίτης Ψάθας - Καλαθοποιίας',
        'Τεχνίτης Ψηφιδωτών',
        'Τεχνικός - Ειδικός Εφαρμογών Πληροφορικής',
        'Τεχνικός Αερίων Καυσίμων',
        'Τεχνικός Αεροσκαφών - Ειδικός Μηχανοσύνθετων Αεροσκαφών',
        'Τεχνικός Αμπελουργίας - Οινοτεχνίας',
        'Τεχνικός Ανελκυστήρων',
        'Τεχνικός Αρδεύσεων',
        'Τεχνικός Αρτοποιίας - Ζαχαροπλαστικής',
        'Τεχνικός Ασφαλείας Δικτύων Η/Υ - Τεχνικός Δικτύων Υπολογιστών',
        'Τεχνικός Αυτοκινήτων Οχημάτων',
        'Τεχνικός Αυτοματισμών',
        'Τεχνικός Διαχείρισης Συστημάτων και Παροχής Υπηρεσιών Intranet - Internet',
        'Τεχνικός Διαχείρισης και Ανακύκλωσης Αποβλήτων',
        'Τεχνικός Δομικών Έργων',
        'Τεχνικός Ελέγχου Βιομηχανικού και Εργασιακού Περιβάλλοντος',
        'Τεχνικός Ελέγχου Ρύπανσης και Εγκαταστάσεων Αντιρρύπανσης',
        'Τεχνικός Ελέγχου Υλικών',
        'Τεχνικός Επεξεργασίας Γάλακτος',
        'Τεχνικός Εργαλειομηχανών',
        'Τεχνικός Εφαρμογών Ιατρικής Πληροφορικής',
        'Τεχνικός Εφαρμογών Πληροφορικής',
        'Τεχνικός Εφαρμογών Πληροφορικής, Δικτύων και Αυτοματισμού Γραφείου',
        'Τεχνικός Θέρμανσης και Υδραυλικών Εγκαταστάσεων',
        'Τεχνικός Κοσμήματος Παραγωγής',
        'Τεχνικός Μηχανών Θαλάσσης και Αναψυχής',
        'Τεχνικός Οργάνων Μετρήσεων',
        'Τεχνικός Ποτοποιίας - Αποσταγματοποιίας',
        'Τεχνικός Υπαξιωματικός Αεροπορίας',
        'Τεχνικός Ψηφιακής Φωτογραφίας και CD-ROM',
        'Τεχνολόγος - Δασοπόνος Διαχείρισης Φυσικών Πόρων',
        'Τεχνολόγος Ακτινολογίας Ακτινοθεραπείας',
        'Τεχνολόγος Αντιρρύπανσης',
        'Τεχνολόγος Βιομηχανικού Σχεδιασμού',
        'Τεχνολόγος Ενεργειακής Τεχνικής',
        'Τεχνολόγος Ιατρικών Οργάνων',
        'Τεχνολόγος Ιχθυοκαλλιέργειας',
        'Τεχνολόγος Ορυχείων',
        'Τεχνολόγος Πετρελαίου',
        'Τεχνολόγος Τηλεϊατρικής',
        'Τεχνολόγος Τροφίμων',
        'Τεχνολόγος Φυτικής Παραγωγής',
        'Τζακάς',
        'Τζαμάς',
        'Τηλεπαρουσιαστής',
        'Τηλεφωνητής',
        'Τουριστικός Πράκτορας',
        'Τραγουδιστής',
        'Τυπογράφος',
        'Υαλουργός',
        'Υδραυλικός',
        'Υλοτόμος',
        'Υπάλληλος Εξυπηρέτησης Πελατών',
        'Υπάλληλος Τουριστικού Γραφείου – Ειδικός Τουριστικού Πρακτορείου',
        'Υπάλληλος Τράπεζας',
        'Υπαξιωματικός Ενόπλων Δυνάμεων',
        'Υπεύθυνος Ανθρώπινου Δυναμικού',
        'Υπεύθυνος Γραφείου Τύπου',
        'Υπεύθυνος Δημοσίων Σχέσεων',
        'Υπεύθυνος Προμηθειών και Διαχείρισης Αποθήκης',
        'Φαρμακοποιός',
        'Φιλόλογος',
        'Φοντοποιός',
        'Φυσικός',
        'Φυσιοθεραπευτής',
        'Φωτογράφος - Τεχνικός Λήψης Φωτογραφίας - Έγχρωμη Φωτογραφία',
        'Φωτομοντέλο - Μανεκέν',
        'Φύλακας Ασφαλείας',
        'Φύλακας Μουσείων και Αρχαιολογικών Χώρων',
        'Χαράκτης',
        'Χειριστές Βαρέων Μηχανημάτων και Αγροτικών Μηχανημάτων',
        'Χειριστής Μηχανημάτων Κοπής Ξύλου',
        'Χειρούργος',
        'Χημικός',
        'Χημικός Μηχανικός',
        'Χορευτής',
        'Χορογράφος',
        'Χρηματιστής',
        'Χρηματοοικονομικός Σύμβουλος',
        'Χτίστης',
        'Ψαράς',
        'Ψυκτικός - Τεχνικός Εγκαταστάσεων Ψύξης Αερισμού και Κλιματισμού',
        'Ψυχίατρος',
        'Ψυχολόγος',
        'Ωκεανογράφος',
        'Ωρολογοποιός',
    ]



# coding=utf-8
from __future__ import unicode_literals
from collections import OrderedDict
from faker.utils.decorators import lowercase, slugify
from .. import Provider as InternetProvider


class Provider(InternetProvider):
    """
    Provider for internet stuff for en_PH locale

    Free email domains are based on anecdotal evidence and experience. Available TLDs are based on the listed sources.
    Because of the local company naming scheme, a custom generator is needed to output convincing company domains.

    Sources:
    - https://en.wikipedia.org/wiki/.ph
    """

    tlds = (
        'com', 'net', 'org', 'ph',
        'com.ph', 'net.ph', 'org.ph',
    )
    safe_email_tlds = tlds
    free_email_domains = (
        'gmail.com',
        'yahoo.com',
        'zohomail.com',
    )
    email_formats = OrderedDict([
        ('{{user_name}}@{{domain_name}}', 0.75),
        ('{{user_name}}@{{free_email_domain}}', 0.25),
    ])

    @lowercase
    @slugify
    def domain_word(self):
        check = self.random_int(0, 99)
        if check % 100 < 40:
            company_acronym = self.generator.format('random_company_acronym')
            if len(company_acronym) == 2:
                company_type = self.generator.format('company_type')
                return company_acronym + company_type
            else:
                return company_acronym
        else:
            if check % 2 == 0:
                name_part = self.generator.format('last_name')
            else:
                name_part = self.generator.format('random_company_adjective')
            company_noun_chain = self.generator.format('random_company_noun_chain')
            company_nouns = company_noun_chain.split(' ')
            if len(company_nouns) == 1:
                return name_part + company_noun_chain
            else:
                company_type = self.generator.format('company_type')
                company_elements = [name_part] + company_nouns
                acronym = ''.join([word[0] for word in company_elements])
                return acronym + company_type



# coding=utf-8

from __future__ import unicode_literals
from .. import Provider as LoremProvider


class Provider(LoremProvider):
    word_list = (
        'تاريخ', 'ثمّة', 'ويكيبيديا', 'واشتدّت', 'عن', 'أوروبا',
        'التغييرات', 'كثيرة', 'سنغافورة', 'اللازمة', 'جُل', 'قد', 'أن',
        'الشّعبين', 'للجزر', 'بالشرقي', 'ذات', 'والتي', 'نهاية',
        'عرفها', 'وسفن', 'وبولندا', 'بالمطالبة', 'وقوعها', 'معاملة',
        'العصبة', 'لمّ', 'البرية', 'بعدم', 'واُسدل', 'حالية', 'فمرّ',
        'قبضتهم', 'وإقامة', 'الأعمال', 'معارضة', 'الإمتعاض', '٢٠٠٤',
        'كانت', 'بينما', 'شعار', 'وتزويده', 'معقل', 'عشوائية', 'مدينة',
        'وفي', 'الأوروبيّون', 'فكان', 'ضرب', 'سكان', 'الأوروبية',
        'الأمريكي', 'وبغطاء', 'الربيع', 'المضي', 'جدول', 'التقليدية',
        'العاصمة', 'التنازلي', 'إبّان', 'بها', 'الجنوبي', 'وتنامت',
        'استدعى', 'الرئيسية', 'بالتوقيع', 'بداية', 'دخول', 'أثره',
        'الدنمارك', 'واقتصار', 'الساحل', 'الوزراء', 'الثالث', 'تونس',
        'الخاسرة', 'الحيلولة', 'لفشل', 'الإطلاق', 'ماذا', 'مواقعها',
        'الجديدة', 'والنفيس', 'الأرواح', 'أكثر', 'الأوربيين',
        'والفرنسي', 'أوزار', 'بلديهما', 'وايرلندا', 'يتم', 'جعل',
        'عرض', 'أفريقيا', 'إستعمل', 'التقليدي', 'عُقر', 'طوكيو',
        'اتفاقية', 'بمحاولة', 'اوروبا', 'منتصف', 'أسر', 'بالرّغم',
        'بضرب', 'إحكام', 'أجزاء', 'الخارجية', 'كنقطة', 'بشرية',
        'بقادة', 'لبلجيكا', 'استطاعوا', 'بسبب', 'تسمّى', 'الحرة',
        'يرتبط', 'شاسعة', 'الواقعة', 'مارد', 'والمعدات', 'لمحاكم',
        'بعلى', 'التبرعات', 'ضمنها', 'فعل', '٣٠', 'الاندونيسية', 'الى',
        'مكثّفة', 'وقدّموا', 'الأثنان', 'أخرى', 'العالمي', 'واحدة',
        'وترك', 'الحكم', 'بهناك', 'بأيدي', 'باستخدام', 'الذود',
        'ولكسمبورغ', 'بعشوائية', 'الأوضاع', 'رجوعهم', 'وأزيز', 'سياسة',
        'ليبين', 'رئيس', 'الجوي', 'سقطت', 'تكاليف', 'وكسبت', 'وعلى',
        'احداث', 'شموليةً', 'إيطاليا', 'عجّل', 'أحكم', 'الدمج',
        'محاولات', 'يعادل', 'بريطانيا-فرنسا', 'ديسمبر', 'قتيل',
        'التاريخ', 'استبدال', 'مقاومة', 'السيء', 'الصعداء', 'أعمال',
        'مسارح', 'ومضى', 'تعد', 'صفحة', 'النفط', 'وهولندا',
        'وبريطانيا', 'جهة', 'بين', 'اتّجة', 'واعتلاء', 'ومحاولة',
        'المارق', 'المؤلّفة', 'شمال', 'لهذه', 'بحث', 'وانهاء', 'غريمه',
        'وبحلول', 'وقبل', 'بتصفح', 'بال', 'الفرنسي', 'وسمّيت', 'ترتيب',
        'واستمرت', 'ببلا', 'الدولارات', 'إعمار', 'إعادة', 'به',
        'الضغوط', 'حصدت', 'بقسوة', 'الطريق', 'وانتهاءً', 'حول',
        'اكتوبر', 'الخاسر', 'ايطاليا', 'للأراضي', 'عالمية', 'جديدة',
        'أخر', 'غينيا', 'قُدُماً', 'و', 'مسؤولية', 'ابتدعها', 'أمّا',
        'الأول', 'أدوات', 'يطول', 'تُصب', 'الأولى', 'المسرح', 'بهيئة',
        'الخطّة', 'موالية', 'مليون', 'بالجانب', 'إيو', 'الهجوم',
        'أعلنت', 'المدن', 'علاقة', 'باستحداث', 'انتباه', 'بالولايات',
        'مدن', 'أساسي', 'لفرنسا', 'الجنوب', 'الثقيل', 'والعتاد',
        'بقعة', 'العدّ', 'اتفاق', 'تحت', 'الإحتفاظ', 'بوقامت', 'التي',
        'ب٠٨٠٤', 'ولاتّساع', 'الا', 'حكومة', 'مما', 'بحيث', 'مليارات',
        'شواطيء', 'التحالف', 'الأوروبي', 'قام', 'يتعلّق', 'مايو',
        'هاربر', 'العمليات', 'أما', 'يعبأ', 'وشعار', 'الإقتصادية',
        'والكساد', 'ليرتفع', 'وبالرغم', 'قِبل', 'الله', 'تكتيكاً',
        'اعتداء', 'ب٣٠', 'تسبب', 'للحكومة', 'الأهداف', 'اللا', 'ببعض',
        'الستار', 'وبلجيكا', 'وفنلندا', 'ميناء', 'الساحلية', 'مقاطعة',
        'وتم', 'السيطرة', 'تغييرات', 'ووصف', 'الجنود', 'بلمّ', 'لدحر',
        'بالحرب', 'كُلفة', 'وبدون', 'وصل', 'أفاق', 'الأبرياء',
        'مهمّات', 'العظمى', 'والقرى', 'وأكثرها', 'لان', 'مشارف',
        'مساعدة', 'أضف', 'مشروط', 'الكونجرس', 'للصين', 'الشمل',
        'اليميني', 'فكانت', 'سقوط', 'بتحت', 'بكلا', 'تصرّف', 'الأثناء',
        'بولندا', 'واندونيسيا', 'كلّ', 'إحتار', 'الخاطفة', 'كردة',
        'وحلفاؤها', 'إتفاقية', 'وبداية', 'يذكر', 'حادثة', 'مئات',
        'الأسيوي', 'غير', 'الانجليزية', 'استعملت', 'استمرار', 'المحيط',
        'وصافرات', 'وحتّى', 'الخاصّة', 'وزارة', 'بحق', 'بدارت',
        'ماليزيا', 'جزيرتي', 'اليابان', 'هامش', 'بوابة', 'وجهان',
        'وقرى', 'تجهيز', 'بولم', 'البولندي', 'الجو', 'لكل', 'فرنسية',
        'تحرّكت', 'يقوم', 'ولم', 'للإتحاد', 'الغالي', 'المشترك',
        'التكاليف', 'بولاتّساع', 'الفرنسية', 'بتحدّي', 'غرّة', 'الأرض',
        'مشاركة', 'فصل', 'بأم', 'والكوري', 'وقد', 'بـ', 'كل', 'بزمام',
        'أي', 'ماشاء', 'لغات', 'اليابانية', 'ونتج', 'الضروري', 'مع',
        'عقبت', 'لليابان', 'مسرح', 'فقامت', 'بتطويق', 'بتخصيص',
        'الوراء', 'تعداد', 'فبعد', 'تشكيل', 'الموسوعة', 'لكون',
        'واستمر', 'الأمريكية', 'إجلاء', 'بدون', 'قررت', 'الشتاء',
        'يتمكن', 'التخطيط', 'مكن', 'بريطانيا', 'أحدث', 'تشيكوسلوفاكيا',
        'وبدأت', 'الشهيرة', 'حدى', 'الإيطالية', 'يتسنّى', 'اقتصادية',
        'التجارية', 'بشكل', 'عسكرياً', 'على', 'ألمّ', 'لبولندا',
        'وسوء', 'جنوب', 'بالشتاء', 'الفترة', 'للمجهود', 'أسيا',
        'ارتكبها', 'بلها', 'قامت', 'أوراقهم', 'بجسيمة', 'بإعمار',
        'وتتحمّل', 'القوى', 'قبل', 'دنو', 'عدم', 'دار', 'الهادي',
        'خطّة', 'المجتمع', 'أملاً', 'دفّة', 'بخطوط', 'السفن', 'لعدم',
        'حاملات', 'بل', 'الإتفاقية', 'الأمور', 'بانه', 'مرجع',
        'النزاع', 'الدّفاع', 'بكلّ', 'ان', 'السبب', 'المشتّتون',
        'ودول', 'ألمانيا', 'والروسية', 'والفلبين', 'بلا', 'أواخر',
        'ببه', 'تزامناً', 'أطراف', 'يونيو', 'الأراضي', 'كانتا',
        'ويعزى', 'انتهت', 'بالثالث', 'وباستثناء', 'زهاء', 'الآخر',
        'أخذ', 'ليركز', 'يتبقّ', 'تمهيد', 'الإقتصادي', 'بالرغم',
        'سبتمبر', 'بقد', 'واتّجه', 'وصغار', 'الشتوية', 'سابق', 'بعض',
        'الشرق', 'المتساقطة', 'الصين', 'اسبوعين', 'لهيمنة', 'الصفحات',
        'أم', 'الساحة', 'القادة', 'لعملة', 'أمام', 'تحرير', 'عليها',
        'وحرمان', 'حلّت', 'بأضرار', 'الثقيلة', 'وجزر', 'الأرضية',
        'أراض', 'والحزب', 'وباءت', 'حقول', 'وبالتحديد', 'لإنعدام',
        'اعلان', 'خلاف', 'انه', 'بتونس', 'بمما', 'الأجل', 'أوسع',
        'الآلاف', 'تطوير', 'اليها', 'البشريةً', 'والإتحاد', 'إختار',
        'وبعض', 'بالمحور', 'غضون', 'شرسة', 'السادس', 'الإثنان',
        'لتقليعة', 'الجنرال', 'تصفح', 'ثانية', 'العناد', 'لم',
        'لإعادة', 'سليمان', 'هناك', 'جمعت', 'الباهضة', 'وبعدما', 'في',
        'تنفّس', 'ويتّفق', 'قدما', 'المنتصر', 'جيوب', 'انتصارهم',
        'قادة', 'تحرّك', 'الإمداد', 'الصينية', 'جسيمة', 'تكبّد',
        'وعُرفت', 'استرجاع', 'فشكّل', 'دول', 'فقد', 'الطرفين',
        'بالرّد', 'المعاهدات', 'الأخذ', 'لإعلان', 'كرسي', 'كلا', 'هذا',
        'الإنذار', 'عملية', 'شيء', 'ومن', 'جيما', 'الشرقية', 'بالعمل',
        'ما', 'الولايات', 'حاول', 'جديداً', 'حين', 'العالم', 'بنقطة',
        'خيار', 'بالجوي', 'والديون', 'بقيادة', 'بيكن', 'يبق',
        'ومطالبة', 'أراضي', 'حتى', 'بأراضي', 'معزّزة', 'الثانية',
        'دون', 'بالسيطرة', 'نتيجة', 'دارت', 'بالإنزال', 'أمدها',
        'الشهير', 'من', 'بلاده', 'تم', 'الصفحة', 'ساعة', 'الإكتفاء',
        'يكن', 'عل', 'أصقاع', 'بالأجل', 'تلك', 'إذ', 'الإتحاد',
        'المتحدة', 'إستيلاء', 'بعرض', 'بفصل', 'ممثّلة', 'شدّت', 'كان',
        'هنا؟', 'المبرمة', 'انذار', 'تعديل', 'ليتسنّى', 'وتنصيب',
        'قائمة', 'الإنزال', 'إعلان', 'للسيطرة', 'بعد', 'بالسادس',
        'العالمية', 'ببحشد', 'بحشد', 'إنطلاق', 'والنرويج', 'هُزم',
        'فاتّبع', 'لها', 'بأسر', 'الشرقي', 'المواد', 'الشمال',
        'الأحمر', '٠٨٠٤', 'ثم', 'نفس', 'وقام', 'عدد', 'أسابيع', 'هو',
        'كما', 'وحتى', 'المتاخمة', 'أهّل', 'أدنى', 'بفرض', 'وبعد',
        'المتّبعة', 'بقصف', 'الدول', 'لأداء', 'جورج', 'وفرنسا',
        'فرنسا', 'هذه', 'الحدود', 'الشطر', 'نقطة', 'بدول', 'فهرست',
        'حيث', 'استراليا', 'وإيطالي', 'دأبوا', 'الحكومة', 'الأمم',
        'فسقط', 'مرمى', 'ذلك', 'مكّن', 'وإعلان', 'بمباركة', 'والمانيا',
    )



# coding=utf-8
from __future__ import unicode_literals
from .. import Provider as LoremProvider


class Provider(LoremProvider):
    """
    Provider for generating Filipino lorem content

    Word list is based on the link below with some filtering, de-conjugating, and additional common words.

    Sources:
    - https://1000mostcommonwords.com/1000-most-common-filipino-words/
    """
    word_list = (
        'abo', 'abot', 'aga', 'agham', 'akin', 'aklat', 'ako', 'akyat', 'alam', 'alang-alang', 'alikabok', 'alin',
        'alok', 'alon', 'ama', 'amin', 'amoy', 'anak', 'ang', 'angkop', 'anim', 'ano', 'antas', 'apat', 'aral', 'araw',
        'ari-arian', 'asa', 'asawa', 'asin', 'aso', 'asukal', 'asul', 'at', 'atin', 'away', 'ay', 'ayos', 'baba',
        'babae', 'babaw', 'bagal', 'bagaman', 'bagay', 'bago', 'bagyo', 'bahagi', 'bahay', 'baka', 'bakal', 'bakasyon',
        'bakit', 'bakuran', 'balat', 'balde', 'balikat', 'banat', 'banda', 'bangka', 'bangko', 'bansa', 'barko', 'basa',
        'basehan', 'baso', 'bata', 'batas', 'batay', 'bato', 'bawas', 'bawat', 'bayad', 'bayan', 'baybayin', 'benta',
        'bente', 'berde', 'bibig', 'bigas', 'bigat', 'bigay', 'bigkas', 'bihira', 'bilang', 'bili', 'bilis', 'binata',
        'binibini', 'binti', 'biyahe', 'biyaya', 'boses', 'braso', 'buhangin', 'buhay', 'buhok', 'bukas', 'bulaklak',
        'bundok', 'buntot', 'buo', 'burol', 'butas', 'buti', 'buto', 'buwan', 'daan', 'dagat', 'dagdag', 'dahil',
        'dahilan', 'dala', 'dalaga', 'dalas', 'dalawampu', 'daliri', 'daloy', 'damit', 'damo', 'dapat', 'dati',
        'dating', 'digmaan', 'dilaw', 'disenyo', 'dito', 'doon', 'dugo', 'dyip', 'edad', 'eksakto', 'eksperimento',
        'elemento', 'enerhiya', 'epekto', 'eroplano', 'espesyal', 'estado', 'gabi', 'gala', 'galaw', 'galit', 'gamit',
        'gamot', 'gana', 'ganap', 'ganda', 'gas', 'gastos', 'gatas', 'gawa', 'gawin', 'gilid', 'giliw', 'ginang',
        'ginoo', 'ginto', 'gising', 'gitna', 'gubat', 'guhit', 'gulo', 'gulong', 'gusto', 'haba', 'habang', 'hakbang',
        'halaga', 'halalan', 'halaman', 'haligi', 'halimbawa', 'hambing', 'hanap', 'hanapbuhay', 'hanay', 'handa',
        'hanggan', 'hanggang', 'hangin', 'hardin', 'hati', 'hatid', 'hatol', 'hayop', 'higit', 'hila', 'hilaga',
        'hilera', 'himpapawid', 'hindi', 'hintay', 'hirap', 'hiwa', 'hiwalay', 'hugis', 'hula', 'huli', 'hulog',
        'humantong', 'husay', 'iba', 'ibabaw', 'ibig', 'ibon', 'ilalim', 'ilan', 'ilang', 'ilog', 'ilong', 'industriya',
        'ingay', 'inggit', 'init', 'inom', 'insekto', 'instrumento', 'inumin', 'ipon', 'isa', 'isda', 'isip', 'iskor',
        'isla', 'itim', 'itlog', 'ito', 'iwan', 'iyon', 'kaaway', 'kababaihan', 'kabayo', 'kabuuan', 'kaganapan',
        'kahit', 'kahon', 'kaibigan', 'kailangan', 'kailanman', 'kain', 'kaisa-isa', 'kakaiba', 'kalabit', 'kalagayan',
        'kalahati', 'kalakal', 'kalakalan', 'kalsada', 'kalye', 'kama', 'kamay', 'kampanilya', 'kampo', 'kanin',
        'kanluran', 'kanta', 'kanya', 'kapag', 'kapal', 'kapangyarihan', 'kapantay', 'kapatid', 'kapit-bahay',
        'kapital', 'kapitan', 'kapwa', 'karagatan', 'karamihan', 'karanasan', 'karaniwan', 'karapatan', 'karne',
        'kasalukuyan', 'kasama', 'kasanayan', 'kasangkapan', 'kasaysayan', 'kaso', 'katangian', 'katarungan', 'katawan',
        'katinig', 'katulad', 'katunayan', 'kawal', 'kaya', 'kaysa', 'kayumanggi', 'kilos', 'kinang', 'kinig', 'klase',
        'ko', 'kompanya', 'koponan', 'kopya', 'kotse', 'kuha', 'kulay', 'kumpleto', 'kung', 'kuskos', 'kuwento',
        'laban', 'lagay', 'lagda', 'lago', 'lahat', 'lahi', 'lakad', 'lakas', 'laki', 'lalim', 'lalo', 'laman',
        'lamang', 'lambak', 'lambot', 'lamig', 'landas', 'langis', 'langit', 'langoy', 'lapit', 'larawan', 'laro',
        'lason', 'lawa', 'lawak', 'layag', 'layo', 'leeg', 'libo', 'libre', 'ligaw', 'ligtas', 'liit', 'likas', 'likha',
        'likido', 'likod', 'lima', 'linaw', 'linggo', 'linis', 'linya', 'lipad', 'listahan', 'litaw', 'liwanag',
        'lubid', 'lugar', 'luma', 'lungsod', 'lupa', 'lupon', 'lutas', 'luwag', 'maaari', 'maaga', 'madali', 'maging',
        'maginoo', 'magkano', 'magulang', 'mahal', 'mahalaga', 'mahirap', 'maikli', 'mainam', 'mainit', 'mais',
        'makina', 'mali', 'maliban', 'manatili', 'manggagawa', 'mangyari', 'mangyaring', 'manipis', 'maniwala',
        'mansanas', 'mapa', 'marahil', 'marami', 'mas', 'masa', 'masyado', 'mata', 'may', 'mayroon', 'medyo', 'merkado',
        'mga', 'milyon', 'minahan', 'minuto', 'mukha', 'mula', 'muli', 'mundo', 'musika', 'na', 'naging', 'nais',
        'nakita', 'namin', 'nanay', 'nawala', 'nayon', 'ng', 'ngayon', 'ngipin', 'ngiti', 'ngunit', 'noon', 'numero',
        'oo', 'opisina', 'opo', 'oras', 'orihinal', 'pa', 'paa', 'paaralan', 'pabor', 'pabuya', 'pader', 'pagitan',
        'pakiramdam', 'paksa', 'palagi', 'palapag', 'pamamagitan', 'pamilya', 'panahon', 'panalo', 'pandiwa',
        'pangalan', 'panganib', 'pangarap', 'pangkat', 'pangmaramihang', 'pangngalan', 'pangunahin', 'pantig',
        'panuntunan', 'papel', 'para', 'paraan', 'pareho', 'pares', 'parirala', 'parisukat', 'partido', 'pasa',
        'pasiya', 'pasok', 'patakaran', 'patlang', 'patnubay', 'pato', 'payag', 'pera', 'pigil', 'pilak', 'pili',
        'pindot', 'pinto', 'piraso', 'pito', 'plano', 'port', 'posible', 'posisyon', 'problema', 'produkto', 'proseso',
        'prutas', 'pula', 'puno', 'punta', 'punto', 'pusa', 'puso', 'puti', 'puwang', 'puwersa', 'radyo', 'rehiyon',
        'resulta', 'sa', 'saan', 'sabay', 'sabi', 'sagot', 'sakahan', 'salamat', 'salamin', 'sali', 'salita', 'sama',
        'sampu', 'sandali', 'sang-ayon', 'sangay', 'sanggol', 'sapat', 'sapatos', 'sarili', 'sariwa', 'saya', 'sayaw',
        'sigaw', 'siglo', 'sigurado', 'sikat', 'sila', 'silangan', 'silya', 'simbolo', 'simula', 'singil', 'singsing',
        'sining', 'sira', 'sistema', 'siya', 'siyam', 'siyempre', 'solusyon', 'subok', 'sukat', 'sulat', 'sulok',
        'sulong', 'sumbrero', 'sundin', 'sundo', 'sunod', 'sunog', 'suot', 'suporta', 'suri', 'taas', 'taba', 'tagal',
        'tagumpay', 'tahanan', 'tahimik', 'tainga', 'takbo', 'takot', 'tala', 'talakay', 'talim', 'talo', 'talon',
        'tama', 'tandaan', 'tanggap', 'tanghali', 'tangi', 'tangkad', 'tanong', 'tao', 'taon', 'tapang', 'tapat',
        'tapon', 'tapos', 'tatlon', 'tatsulok', 'tawag', 'tawid', 'tayo', 'temperatura', 'timbang', 'timog', 'tinapay',
        'tinda', 'tindahan', 'tingin', 'tipon', 'tiyak', 'tono', 'totoo', 'trabaho', 'trak', 'tren', 'tubig', 'tugon',
        'tukoy', 'tuktok', 'tula', 'tulad', 'tulog', 'tulong', 'tuloy', 'tumba', 'tunay', 'tungkol', 'tungo', 'tunog',
        'turo', 'tuwa', 'tuwid', 'ugat', 'ulan', 'ulo', 'una', 'upo', 'upuan', 'uri', 'wala', 'walo', 'wika', 'yaman',
        'yelo',
    )



# coding=utf-8

from __future__ import unicode_literals
from ..es import Provider as AddressProvider


class Provider(AddressProvider):
    building_number_formats = ('%', '%#', '%#', '%#', '%##')
    street_prefixes = (
        'Plaza', 'Calle', 'Avenida', 'Via', 'Vial', 'Rambla', 'Glorieta',
        'Urbanización', 'Callejón', 'Cañada', 'Alameda', 'Acceso', 'C.',
        'Ronda', 'Pasaje', 'Cuesta', 'Pasadizo', 'Paseo', 'Camino',
    )
    postcode_formats = ('#####', )
    states = (
        'Álava',
        'Albacete',
        'Alicante',
        'Almería',
        'Asturias',
        'Ávila',
        'Badajoz',
        'Baleares',
        'Barcelona',
        'Burgos',
        'Cáceres',
        'Cádiz',
        'Cantabria',
        'Castellón',
        'Ceuta',
        'Ciudad',
        'Córdoba',
        'Cuenca',
        'Girona',
        'Granada',
        'Guadalajara',
        'Guipúzcoa',
        'Huelva',
        'Huesca',
        'Jaén',
        'La Coruña',
        'La Rioja',
        'Las Palmas',
        'León',
        'Lleida',
        'Lugo',
        'Madrid',
        'Málaga',
        'Melilla',
        'Murcia',
        'Navarra',
        'Ourense',
        'Palencia',
        'Pontevedra',
        'Salamanca',
        'Santa Cruz de Tenerife',
        'Segovia',
        'Sevilla',
        'Soria',
        'Tarragona',
        'Teruel',
        'Toledo',
        'Valencia',
        'Valladolid',
        'Vizcaya',
        'Zamora',
        'Zaragoza')

    city_formats = (
        '{{state_name}}',
    )

    street_name_formats = (
        '{{street_prefix}} {{first_name}} {{last_name}}',
        '{{street_prefix}} de {{first_name}} {{last_name}}',

    )
    street_address_formats = (
        '{{street_name}} {{building_number}}',
        '{{street_name}} {{building_number}} {{secondary_address}} ',
    )
    address_formats = (
        "{{street_address}}\n{{city}}, {{postcode}}",
    )
    secondary_address_formats = ('Apt. ##', 'Piso #', 'Puerta #')

    def state_name(self):
        return self.random_element(self.states)

    def street_prefix(self):
        return self.random_element(self.street_prefixes)

    def secondary_address(self):
        return self.numerify(
            self.random_element(
                self.secondary_address_formats))

    def state(self):
        return self.random_element(self.states)
